<?php

/**
 * update.php
 * ----------
 * @version 5.4.0
 * @copyright 2002-2010 Melbis Company
 * @link http://www.melbis.com
 * @author Dmitriy Kasyanov
 **/

// Define script folder 
define('FOLDER', './../');

 
// Require classes and includes
require_once('./privat/privat.php');
require_once('./includes/common.php');
require_once('./includes/admin.php');
require_once('./class/show/class.XML.php');
require_once('./class/util/class.PclZip.php');
require_once('./class/util/class.Emailer.php');


// Set time limit 
@set_time_limit(3000); 

// Start session
session_start();


// What you need?
switch ($_POST['command'])
{
	case 'GET_DIR_FILES' : 
		get_dir_files(); 
		break;
	case 'KILL_FILES' : 
		kill_files(); 
		break;
	case 'DO_INSTRUCTIONS' : 
		do_instructions(); 
		break;
	case 'LOCK_SHOP' : 
		lock_shop(); 
		break;		
	case 'UNLOCK_SHOP' : 
		unlock_shop(); 
		break;		
	case 'UNPACK_FILES' : 
		unpack_files(); 
		break;
	case 'GET_STARTUP_INFO' : 
		get_startup_info(); 
		break;		
	case 'GET_PRICE_LIST' : 
		get_price_list(); 
		break;		
	default : 
		echo "PHP_ERROR: Unknow command in line [".__LINE__."] :: Fail Command";
}


/**
 * Function get_dir_files
 * Return server parameters from file privat.php
 **/
function get_dir_files()
{
	list ($start_from, $how) = explode(';', $_POST['values']);
	
	$file_list = '';
	$handle = opendir('./../files');
	clearstatcache();
	$num = 0;
	while ( $file = readdir($handle) )
	{ 
		if ( $file != "." && $file != ".." )
		{ 
			$num++;
			if ( $start_from > $num ) continue;
			if ( $num > $start_from + $how - 1 ) break;
			$file_list .= "\r\n".$file.'='.(filemtime('./../files/'.$file) + date("Z", filemtime('./../files/'.$file)));
		}
	}
	closedir($handle); 
	string_to_zip_answer(ltrim($file_list, "\r\n"));
}


/**
 * Function kill_files
 * Kill old files from server
 **/
function kill_files()
{
	$file_list = explode(';', trim(zip_file_to_string($_FILES['string_file']['tmp_name']), ';'));
	foreach ( $file_list as $file_name) 
	{
		if ( file_exists('./../files/'.$file_name) &&  strlen($file_name) > 0 ) 
		{
			@unlink('./../files/'.$file_name);
		}
	}
	echo "QUERY_EXECUTE";
}

/**
 * Function do_instructions
 * do SQL instructions for update tables
 **/
function do_instructions()
{
	global $gData;
	
	if ( !file_exists($_FILES['data_file']['tmp_name']) ) die("PHP_ERROR: Don't received data file");

  	$Archive = new PclZip($_FILES['data_file']['tmp_name']);
  	if ( $Archive->extract('./tmp') <= 0 ) die("PHP_ERROR: Archive unpack error");

	
	// Connect to Data
	$gData = data_connect(true, array(), true);
	
	// Prepare to update commands
	$gData->PrepareToUpdate();
	
	// Begin transaction
	$gData->BeginTransaction();
	
	// Parsing xml
	$xml_parser = xml_parser_create('UTF-8');
	xml_set_element_handler($xml_parser, "start_xml_tag", "end_xml_tag");
	xml_set_character_data_handler($xml_parser, "xml_tag");
	xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, FALSE);
	$data_file = './tmp/Data.xml';
	if ( !($fp = fopen($data_file, "r")) ) 
	{
		die("PHP_ERROR: Could not open Data.xml");
	}
	while ( $data = fread($fp, filesize($data_file)) ) 
	{
   		if ( !xml_parse($xml_parser, $data, feof($fp)) ) 
		{
       		die(sprintf("PHP_ERROR: XML parsing error: %s at line %d",
                   		xml_error_string(xml_get_error_code($xml_parser)),
                   		xml_get_current_line_number($xml_parser)));
		}
	}
	fclose($fp);
	xml_parser_free($xml_parser); 
	@unlink($data_file);
	
	// Commit transaction
	$gData->CommitTransaction();
	
	echo "QUERY_EXECUTE";
}


/**
 * Function lock_shop
 * make shop temporary unavailable
 **/
function lock_shop()
{
	global $gData, $gOptions;
	
	// Connect to Data
	$gData = data_connect(true, array(), true);	
	
	// Lock
	$gData->SaveShopValue('main_available', 'False');
	
	sleep(3);
	
	echo "QUERY_EXECUTE";
}

/**
 * Function unlock_shop
 * make shop available and do other tasks
 **/
function unlock_shop()
{
	global $gData;
	
	
	// Connect to Data
	$gData = data_connect(true, array(), true);	

	// Save update time
	$gData->SaveShopValue('main_update_time', get_now());
	
	// Set order num if need
	$gOptions = $gData->GetShopValues();
	if ( $gOptions['order_set_num'] == 'True' ) 
	{
	    $gData->SaveShopValue('order_num', $gOptions['order_num_value']*1-1);
		$gData->SaveShopValue('order_set_num', 'False');
	}	
	
	// Reset cache
	$gData->ResetCacheFields();
	
	// Recalc prices
	$currency = $gData->GetCurrency();
	$rows = $gData->GetNumRows($currency);
	for ( $i = 1 ; $i <= $rows; $i++ )
	{
		$hash = $gData->FetchRow($currency);
		$mult = ( $hash[3] == 1 && $hash[2] != 0) ? 1/$hash[2] : $hash[2];
		$gData->MakePriceAndSum($hash[0], $mult);
	}
	
	// Recalc goods voices
	$goods = $gData->GetGoodsVsComment();
	$rows = $gData->GetNumRows($goods);
	for ( $i = 1 ; $i <= $rows; $i++ )
	{
		$hash = $gData->FetchRow($goods);
		$gData->UpdateGoodsVoice($hash[0]);
	}	
		
	// Unlock
	$gData->SaveShopValue('main_available', 'True');
	
	echo "QUERY_EXECUTE";
}


/**
 * Function get_startup_info
 * return current info about goods, orders, etc.
 **/
function get_startup_info()
{
	global $gData;
	
	// Connect to Data
	$gData = data_connect(true);	
	
	// Start XML file
	$xml_file = './tmp/'.md5(session_id()).'.xml';
	$xml = new XML(SHOP_CHARSET, $xml_file);
	$xml->StartTag('MELBISSHOP', array('Version' => 'Melbis Shop v'.SCRIPT_VERSION));
		
	// Request Archive Order
	$sth = $gData->GetTableValues('archive_order', 'id, code, status, get_payment, date_time, inside_note', 'date_time');
	$rows = $gData->GetNumRows($sth);
	$xml->StartTag('ARCHORDER');
	for ( $i = 1; $i <= $rows; $i++ ) 
	{		
		$hash = $gData->FetchHash($sth);
		$hash['date_time'] = date($gOptions['main_fmt_date_time'], strtotime($hash['date_time']));
		$xml->PutTag('ROW', $hash);
	}
	$xml->CloseTag('ARCHORDER');
	
	$xml->CloseTag('MELBISSHOP');
	$xml->Free();
	
	// Put result to archive
	xml_to_zip_answer($xml_file);	

}

/**
 * Function start_xml_tag
 **/
function start_xml_tag($mParser, $mName, $mAttrs) 
{
	global $gDo, $gDoValues, $gCurrentTagName;

   	$gCurrentTagName = $mName;
   	if ( $mName == 'DO' ) 
	{ 
   		$gDo = array();
   		$gDo = $mAttrs;
   		$gDoValues = array();
	} 
	else 
	{
   		$gDoValues[$gCurrentTagName] = '';
	}
}

/**
 * Function xml_tag
 **/
function xml_tag($mParser, $mData) 
{
	global $gDoValues, $gCurrentTagName;
	
 	$gDoValues[$gCurrentTagName] .= iconv("UTF-8", SHOP_CHARSET, str_replace("\r", " ", $mData));
}

/**
 * Function end_xml_tag
 **/
function end_xml_tag($mParser, $mName) 
{
  	global $gDo, $gDoValues, $gData;
    
   	if ($mName == 'DO') 
   	{
	  	// Make currency base fields
	  	if ( $gDo['Operation'] == 'Update' || $gDo['Operation'] == 'Insert' )
	  	{ 
  			switch ($gDo['TableName'])
			{
				case 'store' : 
					if ( isset($gDoValues['price']) ) $gDoValues['price_base'] = $gDoValues['price']; 
					if ( isset($gDoValues['disc_price']) ) $gDoValues['disc_price_base'] = $gDoValues['disc_price']; 
					if ( isset($gDoValues['prov_price']) ) $gDoValues['prov_price_base'] = $gDoValues['prov_price']; 
					break;
				case 'discount' : 
					if ( isset($gDoValues['suma']) ) $gDoValues['suma_base'] = $gDoValues['suma']; 
				   	break;
				case 'param_value' : 
					if ( isset($gDoValues['suma']) ) $gDoValues['suma_base'] = $gDoValues['suma']; 
				   	break;
				case 'store_add' : 
					if ( isset($gDoValues['price']) ) $gDoValues['price_base'] = $gDoValues['price']; 
				   	break;
				case 'condition' : 
					if ( isset($gDoValues['suma']) ) $gDoValues['suma_base'] = $gDoValues['suma']; 
				   	break;
				case 'delivery' : 
					if ( isset($gDoValues['suma']) ) $gDoValues['suma_base'] = $gDoValues['suma']; 
				   	break;
				case 'delivery_cost' : 
					if ( isset($gDoValues['cost']) ) $gDoValues['cost_base'] = $gDoValues['cost']; 
				   	break;
				case 'partner_main' : 
					if ( isset($gDoValues['pay_suma']) ) $gDoValues['pay_suma_base'] = $gDoValues['pay_suma']; 
				   	break;
				case 'partner_click' : 
					if ( isset($gDoValues['suma']) ) $gDoValues['suma_base'] = $gDoValues['suma']; 
				   	break;
			}
	  	}
		// Do operation 
   		$gData->UpdateCommand($gDo, $gDoValues);
   	}
}

/**
 * Function unpack_files
 * unpack files to special shop's folder
 **/
function unpack_files()
{
	if ( !file_exists($_FILES['data_file']['tmp_name']) ) die("PHP_ERROR: Don't received data file");

  	$Archive = new PclZip($_FILES['data_file']['tmp_name']);
  	if ( ($list = $Archive->listContent()) == 0 ) die("PHP_ERROR : Archive error: ".$Archive->errorInfo(true));
  	foreach ( $list as $file ) 
  	{
    	@unlink('./../files/'.$file['filename']);
  	}
  	if ( $list = $Archive->extract('./../files') <= 0 ) die("PHP_ERROR: Archive unpack error");

	echo "QUERY_EXECUTE";
}

/**
 * Function get_price_list
 * Get price list
 **/
function get_price_list()
{
	global $gData;
	
	// Connect to Data
	$gData = data_connect(true, array(), true);	
	
	// Start XML file
	$xml_file = './tmp/'.md5(session_id()).'.xml';
	$xml = new XML(SHOP_CHARSET, $xml_file);
	$xml->StartTag('MELBISSHOP', array('Version' => 'Melbis Shop v'.SCRIPT_VERSION));	
	
	// Request data
	list ($table, $start_from) = explode(';', $_POST['values']);
	list ($table, $field_list, $start, $how_field) = explode(';', $_POST['values']);
	$info = $gData->GetTableValues($table, $field_list);
	$rows = $gData->GetNumRows($info);
	$gData->DataSeek($info, $start - 1);
	$how = $how_field;
	if ( ( $rows - $start + 1 )  < $how )  
	{ 
		$how = ( $rows - $start + 1 ); 
	}
	$xml->StartTag('TABLE', array('total' => $rows, 'trans' => $how));
	for ( $i = 1; $i <= $how; $i++ ) 
	{	
		$xml->PutTag('ROW',  $gData->FetchHash($info));
	}
	$xml->CloseTag('TABLE');
	$xml->CloseTag('MELBISSHOP');
	$xml->Free();
	
	// Put result to archive
	xml_to_zip_answer($xml_file);
}

?>