<?php

/**
 * class.DataBase.php
 * ------------------
 * @version 5.4.0
 * @copyright 2002-2010 Melbis Company
 * @link http://www.melbis.com
 * @author Dmitriy Kasyanov
 **/

/**
 * Class DataBase
 * Operate with data via MySQL SQL-queries.
 **/
class DataBase
{

	/**
	 * Vars
	 **/
	var $gDB;
	var $gDBNick;
	var $gDBHostName;
	var $gDBUserName;
	var $gDBUserPass;
	var $gDBName;
	var $gFileCSV = false;
	var $gCSVCols ='';
	var $gLoadDataAllow = false;
	var $gLoadDataType = 'LOCAL';


	/**
	 * Constructor
	 **/
	function DataBase($mSimpleError, $mParam)
	{
		if ( count($mParam) == 0 )
		{
			$this->gDBNick = DB_NICK;
			$this->gDBHostName = DB_HOST_NAME;
			$this->gDBUserName = DB_USER_NAME;
			$this->gDBUserPass = DB_USER_PASS;
			$this->gDBName = DB_NAME;
		}
		else
		{
			$this->gDBNick = $mParam['DB_NICK'];
			$this->gDBHostName = $mParam['DB_HOST_NAME'];
			$this->gDBUserName = $mParam['DB_USER_NAME'];
			$this->gDBUserPass = $mParam['DB_USER_PASS'];
			$this->gDBName = $mParam['DB_NAME'];
		}


		$this->ConnectToDB($mSimpleError);
	}

	/**
	 * Connect
	 **/
	function ConnectToDB($mSimpleError)
	{
		require_once(SHOP_CWD.'/admin/class/data/access/class.MySQL3x.php');

		$this->gDB = new MySQL3x(basename(__FILE__),
								 __LINE__,
								 $mSimpleError,
							     $this->gDBHostName,
							     $this->gDBUserName,
							     $this->gDBUserPass,
							     $this->gDBName
							     );
	}



	/**
	 * GetNumRows
	 **/
	function GetNumRows($mQuery)
	{
		return $this->gDB->GetNumRows($mQuery);
	}

	/**
	 * GetNumFields
	 **/
	function GetNumFields($mQuery)
	{
		return $this->gDB->GetNumFields($mQuery);
	}

	/**
	 * GetFieldName
	 **/
	function GetFieldName($mQuery, $mColNum)
	{
		return $this->gDB->GetFieldName($mQuery, $mColNum);
	}

	/**
	 * FetchHash
	 **/
	function FetchHash($mQuery)
	{
		return $this->gDB->FetchHash($mQuery);
	}

	/**
	 * FetchRow
	 **/
	function FetchRow($mQuery)
	{
		return $this->gDB->FetchRow($mQuery);
	}

	/**
	 * DataSeek
	 **/
	function DataSeek($mQuery, $mPos)
	{
		return $this->gDB->DataSeek($mQuery, $mPos);
	}

	/**
	 * BeginTransaction
	 **/
	function BeginTransaction()
	{
		// $this->gDB->DoQuery(__LINE__, 'BEGIN');
		// MySQL 3.x havn't transactions, so it's false
		return false;
	}

	/**
	 * CommitTransaction
	 **/
	function CommitTransaction()
	{
		// $this->gDB->DoQuery(__LINE__, 'COMMIT');
		// MySQL 3.x havn't transactions, so it's false
		return false;
	}

	/**
	 * RollbackTransaction
	 **/
	function RollbackTransaction()
	{
		// $this->gDB->DoQuery(__LINE__, 'ROLLBACK');
		// MySQL 3.x havn't transactions, so it's false
		return false;
	}


	/**
	 * DoSQLCommand
	 **/
	function DoSQLCommand($mCommand)
	{
		return $this->gDB->DoQuery(__LINE__, $mCommand);
	}

	/************************************************************************************************/


	/**
	 * InstallTables
	 **/
	function InstallTables()
	{
		// require tables info structure
		require_once(SHOP_CWD.'/admin/includes/tables.php');
		require_once(SHOP_CWD.'/admin/includes/messages.php');

		// define fields types, indexes and attributes
		$field_type = array('INT'		=> 'INT',
							'TINYINT'	=> 'TINYINT',
							'BIGINT'	=> 'BIGINT',
							'DECIMAL'	=> 'DECIMAL',
							'SMALLINT'	=> 'SMALLINT',
							'VARCHAR'	=> 'VARCHAR',
							'MEDIUMTEXT'=> 'MEDIUMTEXT',
							'DATE'		=> 'DATE',
							'TIME'		=> 'TIME',
							'DATETIME'  => 'DATETIME',
							'ENUM'		=> 'ENUM'
							);
		$attr_type =  array('NOT NULL'		=> 'NOT NULL',
							'AUTO_INCREMENT'=> 'AUTO_INCREMENT',
							'UNSIGNED'		=> 'UNSIGNED'
							);
		$index_type = array('PRIMARY KEY'	=> 'PRIMARY KEY',
							'KEY'			=> 'KEY'
							);
		// create tables
		foreach ( $gTables as $table )
		{
			// Make Create command
			$command = 'CREATE TABLE `'.$this->gDBNick.'_'.$table['table_name'].'` (';
			// echo fields
			foreach ( $table[fields] as $name => $field )
			{
				$command .= '`'.$name.'` '.$field_type[$field['type']];
				if ( strlen($field[size]) > 0 )
				{
					$command .= '('.$field['size'].')';
				}
				foreach ($field[attr] as $attr)
				{
					$command .= ' '.$attr_type[$attr];
				}
				$command .= ', ';
			}
			// echo indexes
			foreach ( $table[indexes] as $name => $type )
			{
				$command .= $index_type[$type].' ('.$name.'), ';
			}
			$command = substr($command, 0, strlen($command)-2);
			$command .= ')';
			$this->gDB->DoQuery(__LINE__, 'DROP TABLE IF EXISTS `'.$this->gDBNick.'_'.$table['table_name'].'`');
			$this->gDB->DoQuery(__LINE__, $command);
		}

		// Insert base records
		$command = "INSERT INTO {$this->gDBNick}_client
		                        (login, pass)
						 VALUES ('".addslashes(iconv($gMessageCharset, SHOP_CHARSET, $gMessage['default']))."', '".md5(session_id())."')
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_shop_values
		                        (key_name, key_value)
						 VALUES ('order_num', '0')
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_shop_values
		                        (key_name, key_value)
						 VALUES ('main_available', 'True')
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * RepairTables
	 **/
	function RepairTables()
	{
		// require tables info structure
		require_once(SHOP_CWD.'/admin/includes/tables.php');

		foreach ( $gTables as $table )
		{
			// Make command
			$command = 'REPAIR TABLE `'.$this->gDBNick.'_'.$table['table_name'].'`';
			$this->gDB->DoQuery(__LINE__, $command);
		}

		return true;
	}

	/**
	 * Function PrepareToUpdate
	 **/
	function PrepareToUpdate()
	{
	    if ( !$this->gFileCSV )
		{
	        @fclose($this->gFileCSV);
	    }
		$this->gFileCSV = fopen(SHOP_CWD.'/admin/tmp/load_data.csv', "w");

		return true;
	}

	/**
	 * Function UpdateCommand
	 **/
	function UpdateCommand($mDo, $mDoValues)
	{
		$load_data = false;

		// Compile command
		switch ($mDo['Operation'])
		{
			case 'Update' :
				$command = 'UPDATE `'.$this->gDBNick.'_'.$mDo['TableName'].'` SET ';
        		foreach ( $mDoValues as $Field => $Value )
				{
        			$command .= " `$Field`='".addslashes($Value)."',";
		 		}
        		$command = rtrim($command, ',');
				$command .= ' WHERE ';
				$Keys = explode(';', $mDo['KeyNames']);
				$Values = explode(';', $mDo['KeyValues']);
				for ( $i = 0; $i < count($Keys); $i++ )
				{
					$command .= " `$Keys[$i]`='".$Values[$i]."' AND";
				}
				$command = rtrim($command, ' AND');
        		break;
	    	case 'Insert' :
        		if ( $this->gLoadDataAllow )
        		{
        			$command = '';
        			$this->gCSVCols = '';
					$data_csv = '';
					foreach ( $mDoValues as $Field => $Value )
					{
        				$data_csv .= addslashes(str_replace("\t", " ", $Value))."\t";
        				$this->gCSVCols .= '`'.$Field.'`, ' ;
        			}
        			$data_csv = rtrim($data_csv , '\t')."\r\n";
        			fputs($this->gFileCSV, $data_csv, strlen($data_csv));
        		}
        		else
        		{
        			$command = 'INSERT INTO `'.$this->gDBNick.'_'.$mDo['TableName'].'` SET ';
        			foreach ( $mDoValues as $Field => $Value )
					{
        				$command .= " `$Field`='".addslashes($Value)."',";
		 			}
        			$command = rtrim($command, ',');
        		}
        		break;
	    	case 'LoadData' :
				if ( $this->gLoadDataAllow )
				{
	    			$this->gCSVCols = rtrim($this->gCSVCols , ', ');
        			$command = 'LOAD DATA '.$this->gLoadDataType.' INFILE \''.
        			str_replace('\\', '/', SHOP_CWD.'/admin/tmp/load_data.csv').
        			'\' REPLACE INTO TABLE `'.$this->gDBNick.'_'.$mDo['TableName'].
        			'` FIELDS TERMINATED BY \'\t\' ENCLOSED BY \'"\' ESCAPED BY \'\\\\\' LINES TERMINATED BY \'\r\n\''.
        			' ('.$this->gCSVCols.')';
					$load_data = true;
				}
        		break;
	    	case 'Delete' :
				$command = 'DELETE FROM `'.$this->gDBNick.'_'.$mDo['TableName'].'` WHERE ';
				$Keys = explode(';', $mDo['KeyNames']);
				$Values = explode(';', $mDo['KeyValues']);
				for ( $i = 0; $i < count($Keys); $i++ )
				{
					$command .= " `$Keys[$i]`='".$Values[$i]."' AND";
				}
				$command = rtrim($command, ' AND');
        		break;
	        case 'Clear'  :
				$command = 'DELETE FROM '.$this->gDBNick.'_'.$mDo['TableName'];
				if ( $mDo['TableName'] == 'shop_values' )
				{
				    $command .= ' WHERE key_name <> "order_num" AND key_name <> "main_available"';
				}
        		break;
		}
		// Do command
		if ( strlen($command) > 0 )
		{
			/*
			$fsql = fopen('./Data.sql', "a");
			fputs($fsql, $command.";\r\n", strlen($command.";\r\n"));
			fclose($fsql);
			*/
			$this->gDB->DoQuery(__LINE__, $command);
			if ( $load_data )
			{
			    $this->PrepareToUpdate();
				$load_data = false;
			}
		}

		return true;
	}


	/**
	 * UnLockTables
	 **/
	function UnLockTables()
	{
		$command = "UNLOCK TABLES";
	   	$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}


	//***************************************************************************************************
	//
	// 									   		Cron
	//
	//***************************************************************************************************

	/**
	 * ClearStat
	 **/
	function ClearStat($mYesterday, $mOneMonth, $mOneYear, $mTotalDays)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_shop_stat
					 WHERE date_time < '$mYesterday'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_visitor
					 WHERE isday < '$mTotalDays'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_visitor_day
					 WHERE isday < '$mOneMonth'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_visitor_week
					 WHERE isweek < '$mOneYear'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_visitor_month
					 WHERE ismonth < '$mOneYear'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_topic
					 WHERE isday < '$mTotalDays'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_store
					 WHERE isday < '$mTotalDays'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_store_buy
					 WHERE isday < '$mTotalDays'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_search
					 WHERE isday < '$mTotalDays'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_ss_order
					 WHERE isweek < '$mTotalDays'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetOldClient
	 **/
	function GetOldClient($mToDay)
	{
		$command = "SELECT id
					  FROM {$this->gDBNick}_client
					 WHERE last_date < '$mToDay'
		               AND id > 1
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearOldClient
	 **/
	function ClearOldClient($mClientID)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_client_field_value
					 WHERE client_id = '$mClientID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_client
					 WHERE id = '$mClientID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetOldArchOrder
	 **/
	function GetOldArchOrder($mToDay)
	{
		$command = "SELECT id
					  FROM {$this->gDBNick}_archive_order
					 WHERE date_time < '$mToDay 23:59:59'
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearOldArchOrder
	 **/
	function ClearOldArchOrder($mOrderID)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_archive_client_field
					 WHERE order_id = '$mOrderID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_archive_basket
					 WHERE order_id = '$mOrderID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE
					  FROM {$this->gDBNick}_archive_order
					 WHERE id = '$mOrderID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetGoodsForRemember
	 **/
	function GetGoodsForRemember()
	{
		$command = "SELECT remember.*, store.name, IF(store_add.status IS NULL, 0, 1) AS param_exists,
						   store.how AS store_how, store.status AS store_status,
						   store_add.how AS store_add_how, store_add.status AS store_add_status
					  FROM {$this->gDBNick}_remember remember
		         LEFT JOIN {$this->gDBNick}_store store
		                ON store.id = remember.store_id
		         LEFT JOIN {$this->gDBNick}_store_add store_add
		                ON ( store_add.store_id = remember.store_id	AND store_add.param_value_id = remember.param_value_id )
					 WHERE store.kind <> 1
		               AND store.active = 1
				  ORDER BY remember.date_time
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearForRemember
	 **/
	function ClearForRemember($mEmail, $mGoodsID, $mParamValueID)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_remember
					 WHERE email = '$mEmail'
					   AND store_id = '$mGoodsID'
					   AND param_value_id = '$mParamValueID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * ClearOldRemember
	 **/
	function ClearOldRemember($mToDay)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_remember
					 WHERE date_time < '$mToDay'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}



	/**
	 * GetGhostArchGoods
	 **/
	function GetGhostArchGoods()
	{
		$command = "SELECT DISTINCT archive_basket.order_id
					  FROM {$this->gDBNick}_archive_basket archive_basket
				 LEFT JOIN {$this->gDBNick}_archive_order archive_order
		    		    ON archive_basket.order_id = archive_order.id
					 WHERE archive_order.id IS NULL
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGhostArchClient
	 **/
	function GetGhostArchClient()
	{
		$command = "SELECT DISTINCT archive_client_field.order_id
					  FROM {$this->gDBNick}_archive_client_field archive_client_field
				 LEFT JOIN {$this->gDBNick}_archive_order archive_order
		    		    ON archive_client_field.order_id = archive_order.id
					 WHERE archive_order.id IS NULL
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGhostClientField
	 **/
	function GetGhostClientField()
	{
		$command = "SELECT DISTINCT client_field_value.client_id
					  FROM {$this->gDBNick}_client_field_value client_field_value
				 LEFT JOIN {$this->gDBNick}_client client
		    		    ON client_field_value.client_id = client.id
					 WHERE client.id IS NULL
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGhostNews
	 **/
	function GetGhostNews()
	{
		$command = "SELECT DISTINCT subscriber.news_id
					  FROM {$this->gDBNick}_subscriber subscriber
				 LEFT JOIN {$this->gDBNick}_news news
		    		    ON subscriber.news_id = news.id
					 WHERE news.id IS NULL
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearSubscriber
	 **/
	function ClearSubscriber($mNewsID)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_subscriber
					 WHERE news_id = $mNewsID
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetGhostVote
	 **/
	function GetGhostVote()
	{
		$command = "SELECT DISTINCT vote_pool.vote_list_id
					  FROM {$this->gDBNick}_vote_pool vote_pool
				 LEFT JOIN {$this->gDBNick}_vote_list vote_list
		    		    ON vote_pool.vote_list_id = vote_list.id
					 WHERE vote_list.id IS NULL
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearVotePool
	 **/
	function ClearVotePool($mVoteListID)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_vote_pool
					 WHERE vote_list_id = $mVoteListID
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetGhostComment
	 **/
	function GetGhostComment()
	{
		$command = "SELECT DISTINCT store_comment.store_id
					  FROM {$this->gDBNick}_store_comment store_comment
				 LEFT JOIN {$this->gDBNick}_store store
		    		    ON store_comment.store_id = store.id
					 WHERE store.id IS NULL
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearComment
	 **/
	function ClearComment($mStoreID)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_store_comment
					 WHERE store_id = '$mStoreID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * ClearPartnerStat
	 **/
	function ClearPartnerStat($mDateToFull, $mDateToDays, $mDateToMonth, $mLastPartDate, $mAccountYear)
	{
		$command = "DELETE FROM {$this->gDBNick}_partner_stat
					 WHERE date_time < '$mDateToFull'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_stat_days
					 WHERE isday < '$mDateToDays'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_stat_month
					 WHERE ismonth < '$mDateToMonth'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_stat_common
					 WHERE isday < '$mDateToDays'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner
					      WHERE last_date < '$mLastPartDate'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_account
					      WHERE date_time < '$mAccountYear'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return  true;
	}

	/**
	 * GetGhostPartInfo
	 **/
	function GetGhostPartInfo($mTable)
	{
		$command = "SELECT DISTINCT $mTable.partner_id
					  FROM {$this->gDBNick}_$mTable $mTable
				 LEFT JOIN {$this->gDBNick}_partner partner
		    		    ON $mTable.partner_id = partner.id
					 WHERE partner.id IS NULL
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearPartnerInfo
	 **/
	function ClearPartnerInfo($mTable, $mPartnerID)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_$mTable
					 WHERE partner_id = '$mPartnerID'
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}


	//***************************************************************************************************
	//
	// 									   		Common
	//
	//***************************************************************************************************


	/**
	 * GetTableValues
	 **/
	function GetTableValues($mTable, $mFields, $mOrderFields = '', $mOrderDesc = false, $mLimit = '')
	{
		$command = "SELECT $mFields
		    		  FROM {$this->gDBNick}_$mTable
				  ";
		if ( strlen($mOrderFields) > 0 ) $command .= 'ORDER BY '.$mOrderFields;
		if ( $mOrderDesc ) $command .= ' DESC';
		if ( strlen($mLimit) > 0 ) $command .= ' LIMIT '.$mLimit;

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetTableStatus
	 **/
	function GetTableStatus()
	{
		$command = "SHOW TABLE STATUS";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClearTable
	 **/
	function ClearTable($mTable)
	{
		$command = "DELETE FROM {$this->gDBNick}_$mTable";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * RestoreTableData
	 **/
	function RestoreTableData($mFields, $mTable, $mKeyField, $mUpdateField)
	{
		if ( count($mUpdateField) != 0 )
		{
			$update_fields = '';
			foreach ( $mUpdateField as $field )
			{
				$update_fields .= '`'.$field.'`=\''.addslashes($mFields[$field]).'\',';
			}
			$update_fields = rtrim($update_fields, ',');

			$key_fields = '';
			foreach ( $mKeyField as $field )
			{
				$key_fields .= '`'.$field.'`=\''.addslashes($mFields[$field]).'\' AND';
			}
			$key_fields = rtrim($key_fields, ' AND');

			$command = "UPDATE {$this->gDBNick}_$mTable
						   SET $update_fields
						 WHERE $key_fields
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}
		else
		{
			$insert_fields = '';
			foreach ( $mFields as $field => $value )
			{
				$insert_fields .= '`'.$field.'`=\''.addslashes(iconv("UTF-8", SHOP_CHARSET, $value)).'\',';
			}
			$insert_fields = rtrim($insert_fields, ',');

			$command = "INSERT INTO {$this->gDBNick}_$mTable
						   SET $insert_fields
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}
	}

	/**
	 * SaveShopStat
	 **/
	function SaveShopStat($mSSID, $mIP, $mFromPage, $mUserAgent, $mPageCode, $mElemID, $mElemCode, $mDateTime)
	{
		$command = "INSERT INTO {$this->gDBNick}_shop_stat
					   SET sid = '$mSSID',
						   ip = '$mIP',
						   from_page = '$mFromPage',
						   user_agent = '$mUserAgent',
						   page_code = '$mPageCode',
						   elem_id = '$mElemID',
						   elem_code = '$mElemCode',
						   date_time = '$mDateTime'
			   	   ";
	   	$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SaveStatOrder
	 **/
	function SaveStatOrder($mDay, $mWeek, $mMonth, $mFromPage, $mDomen, $mKeyword, $mOrderSum, $mDateTime)
	{

		$command = "INSERT INTO {$this->gDBNick}_ss_order
					   SET ismonth = '$mMonth',
						   isweek = '$mWeek',
						   from_page = '$mFromPage',
						   domen = '$mDomen',
						   keyword = '$mKeyword',
						   suma = '$mOrderSum',
					   	   date_time = '$mDateTime'
		   	   	   ";
   		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SaveStatVisitor
	 **/
	function SaveStatVisitor($mDay, $mWeek, $mMonth, $mVisitor)
	{
		$command = "UPDATE {$this->gDBNick}_ss_visitor_day
					   SET viewer = viewer + 1,
						   visitor = visitor + $mVisitor
					 WHERE isday = '$mDay'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_ss_visitor_day
				   		        SET isday = '$mDay',
									viewer = 1,
						   			visitor = 1
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}

		$command = "UPDATE {$this->gDBNick}_ss_visitor_week
					   SET viewer = viewer + 1,
						   visitor = visitor + $mVisitor
					 WHERE isweek = '$mWeek'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_ss_visitor_week
				   		        SET isweek = '$mWeek',
									viewer = 1,
						   			visitor = 1
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}

		$command = "UPDATE {$this->gDBNick}_ss_visitor_month
					   SET viewer = viewer + 1,
						   visitor = visitor + $mVisitor
					 WHERE ismonth = '$mMonth'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_ss_visitor_month
				   		        SET ismonth = '$mMonth',
									viewer = 1,
						   			visitor = 1
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}

		return true;
	}

	/**
	 * SaveStatVisitorFrom
	 **/
	function SaveStatVisitorFrom($mDay, $mWeek, $mMonth, $mDomen, $mKeyword)
	{

		$command = "UPDATE {$this->gDBNick}_ss_visitor
					   SET how = how + 1
					 WHERE isday = '$mDay'
					   AND isweek = '$mWeek'
					   AND ismonth = '$mMonth'
					   AND domen = '$mDomen'
					   AND keyword = '$mKeyword'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_ss_visitor
					 		    SET isday = '$mDay',
						   			isweek = '$mWeek',
						   			ismonth = '$mMonth',
						   			domen = '$mDomen',
						   			keyword = '$mKeyword',
						   			how = '1'
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}

		return true;
	}

	/**
	 * SaveStatTopic
	 **/
	function SaveStatTopic($mDay, $mWeek, $mMonth, $mTopicID)
	{

		$command = "UPDATE {$this->gDBNick}_ss_topic
					   SET how = how + 1
					 WHERE isday = '$mDay'
					   AND isweek = '$mWeek'
					   AND ismonth = '$mMonth'
					   AND id = '$mTopicID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_ss_topic
						 	    SET isday = '$mDay',
							    	isweek = '$mWeek',
						   			ismonth = '$mMonth',
						   			id = '$mTopicID',
									how = 1
			   	   	   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}

		return true;
	}

	/**
	 * SaveStatSearch
	 **/
	function SaveStatSearch($mDay, $mWeek, $mMonth, $mWord, $mPhrase)
	{
		$command = "INSERT INTO {$this->gDBNick}_ss_search
					   SET isday = '$mDay',
						   isweek = '$mWeek',
						   ismonth = '$mMonth',
						   one_word = '$mWord',
						   phrase = '$mPhrase'
			   	   ";
	   	$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SaveStatGoods
	 **/
	function SaveStatGoods($mDay, $mWeek, $mMonth, $mGoodsID)
	{
		$command = "UPDATE {$this->gDBNick}_ss_store
					   SET how = how + 1
					 WHERE isday = '$mDay'
					   AND isweek = '$mWeek'
					   AND ismonth = '$mMonth'
					   AND id = '$mGoodsID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_ss_store
						 	    SET isday = '$mDay',
							    	isweek = '$mWeek',
						   			ismonth = '$mMonth',
						   			id = '$mGoodsID',
									how = 1
			   	   	   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}

		return true;
	}


	/**
	 * SaveStatGoodsBuy
	 **/
	function SaveStatGoodsBuy($mDay, $mWeek, $mMonth, $mGoodsID, $mGoodsSum)
	{
		$command = "INSERT INTO {$this->gDBNick}_ss_store_buy
					   SET isday = '$mDay',
						   isweek = '$mWeek',
						   ismonth = '$mMonth',
						   id = '$mGoodsID',
						   suma = '$mGoodsSum'
			   	   ";
	   	$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}



	/**
	 * SaveShopValue
	 **/
	function SaveShopValue($mKeyName, $mKeyValue)
	{
		$command = "UPDATE {$this->gDBNick}_shop_values
					   SET key_value = '$mKeyValue'
			   		 WHERE key_name = '$mKeyName'
			   	   ";
	   	$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetShopValues
	 **/
	function GetShopValues()
	{
		$command = "SELECT key_name, key_value
		    	  	  FROM {$this->gDBNick}_shop_values
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		while($hash = $this->FetchRow($sth))
		{
			$options[$hash[0]] = $hash[1];
		}

		return $options;
	}

	/**
	 * GetManFullEmails
	 **/
	function GetManFullEmails()
	{
		$command = "SELECT email_full, charset_full
		    	  	  FROM {$this->gDBNick}_manager
					 WHERE email_full <> ''
				  ORDER BY charset_full
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetManShortEmails
	 **/
	function GetManShortEmails()
	{
		$command = "SELECT email_short, charset_short
		    	  	  FROM {$this->gDBNick}_manager
					 WHERE email_short <> ''
				  ORDER BY charset_short
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetTextBlocks
	 **/
	function GetTextBlocks($mArea)
	{
		$command = "SELECT name, content, place_code, dir_name
		    	  	  FROM {$this->gDBNick}_text_block
					 WHERE $mArea = 1 AND isfolder = 0
				  ORDER BY absindex
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPageProperties
	 **/
	function GetPageProperties($mPageID = 0)
	{
		$command = "SELECT *
	    	  	  	  FROM {$this->gDBNick}_topic
				     WHERE id = '$mPageID'
			       ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchHash($sth);

		return $hash;
	}

	/**
	 * FindTopicPSU
	 **/
	function FindTopicPSU($mPSU)
	{
		$command = "SELECT id
	    	  	  	  FROM {$this->gDBNick}_topic
				     WHERE seo_psu = '$mPSU'
			       ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0]*1;
	}

	/**
	 * GetMainPageProperties
	 **/
	function GetMainPageProperties()
	{
		$command = "SELECT *
	    	  	  	  FROM {$this->gDBNick}_topic
				     WHERE istype = 3
				       ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetDiscountOrder
	 **/
	function GetDiscountOrder($mItogoSum)
	{
		$command = "SELECT *
	    	  	  	  FROM {$this->gDBNick}_discount_order
				     WHERE from_sum <= '$mItogoSum'
				  ORDER BY from_sum DESC
				     LIMIT 1
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetCategoryName
	 **/
	function GetCategoryName($mCatID)
	{
		$command = "SELECT id, name
	    	  	  	  FROM {$this->gDBNick}_category
				     WHERE id = '$mCatID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchRow($sth);
	}

	/**
	 * SaveRemember
	 **/
	function SaveRemember($mEmail, $mStoreID, $mParamValueID, $mDateTime)
	{
		$command = "SELECT COUNT(*)
	    	  	  	  FROM {$this->gDBNick}_remember
				     WHERE email = '$mEmail'
					   AND store_id = '$mStoreID'
					   AND param_value_id = '$mParamValueID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);
		if ( $hash[0] > 0 ) return false;

		$command = "INSERT INTO {$this->gDBNick}_remember
				            SET email = '$mEmail',
					            store_id = '$mStoreID',
					   			param_value_id = '$mParamValueID',
								date_time = '$mDateTime'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetMessenger
	 **/
	function GetMessenger()
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_messenger
				  ORDER BY pos
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}


	//***************************************************************************************************
	//
	// 									   Recalc after upload functions
	//
	//***************************************************************************************************


	/**
	 * GetCurrency
	 **/
	function GetCurrency()
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_currency
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * UpdateCurrency
	 **/
	function UpdateCurrency($mName, $mMult)
	{
		$command = "UPDATE {$this->gDBNick}_currency
					   SET multiplex = '$mMult'
					 WHERE name = '$mName'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * ResetCacheFields
	 **/
	function ResetCacheFields()
	{
		$command = "UPDATE {$this->gDBNick}_store
					   SET cache_base_param = -1,
					       cache_info_preview = -1,
						   cache_family = -1,
						   cache_param = -1,
						   cache_disc = -1,
						   cache_advert = -1
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}


	/**
	 * MakePriceAndSum
	 **/
	function MakePriceAndSum($mCurrID, $mCurrMultiplex)
	{
		$command = "UPDATE {$this->gDBNick}_store
					   SET price = price_base * $mCurrMultiplex
					 WHERE price_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_store
					   SET disc_price = disc_price_base * $mCurrMultiplex
					 WHERE dprice_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_store
					   SET prov_price = prov_price_base * $mCurrMultiplex
					 WHERE pprice_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_discount
					   SET suma = suma_base * $mCurrMultiplex
					 WHERE suma_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_param_value
					   SET suma = suma_base * $mCurrMultiplex
					 WHERE suma_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_store_add
					   SET price = price_base * $mCurrMultiplex
					 WHERE price_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_condition
					   SET suma = suma_base * $mCurrMultiplex
					 WHERE suma_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_delivery
					   SET suma = suma_base * $mCurrMultiplex
					 WHERE suma_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_delivery_cost
					   SET cost = cost_base * $mCurrMultiplex
					 WHERE cost_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_partner_click
					   SET suma = suma_base * $mCurrMultiplex
					 WHERE suma_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "UPDATE {$this->gDBNick}_partner_main
					   SET pay_suma = pay_suma_base * $mCurrMultiplex
					 WHERE pay_suma_curr_id = $mCurrID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetGoodsVsComment
	 **/
	function GetGoodsVsComment()
	{
		$command = "SELECT DISTINCT store_id
		    	  	  FROM {$this->gDBNick}_store_comment
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}


	//***************************************************************************************************
	//
	// 									   Goods relative information
	//
	//***************************************************************************************************

	/**
	 * GetInfoGoods
	 **/
	function GetInfoGoods($mGoodsID, $mArea)
	{
		$command = "SELECT info.id, info.name, info.tlevel, info.key_name, info.isfolder, info_value.name,
						   info.tindex, info_value.id, LENGTH(info.descr)
		    		  FROM {$this->gDBNick}_info info
			     LEFT JOIN {$this->gDBNick}_store_info store_info
				        ON store_info.info_id = info.id AND store_info.store_id = '$mGoodsID'
				 LEFT JOIN {$this->gDBNick}_info_value info_value
				        ON store_info.value_id = info_value.id
		   			 WHERE info.$mArea = 1
					   AND ( info.isfolder = 1 OR info_value.name IS NOT NULL )
				  ORDER BY info.absindex DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetInfoValueGoods
	 **/
	function GetInfoValueGoods($mGoodsID, $mInfoID)
	{
		$command = "SELECT info_value.name
		    		  FROM {$this->gDBNick}_info_value info_value,
						   {$this->gDBNick}_store_info store_info
		   			 WHERE store_info.store_id =  $mGoodsID
					   AND store_info.info_id = $mInfoID
					   AND store_info.value_id = info_value.id

					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) > 0 )
		{
			$hash = $this->FetchRow($sth);
			return $hash[0];
		}

		return '';
	}

	/**
	 * GetInfoDescr
	 **/
	function GetInfoDescr($mInfoID)
	{
		$command = "SELECT descr
		    	  	  FROM {$this->gDBNick}_info
					 WHERE id = $mInfoID
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) > 0 )
		{
			$hash = $this->FetchRow($sth);
			return $hash[0];
		}

		return '';
	}

	/**
	 * GetInfoAllKeyName
	 **/
	function GetInfoAllKeyName($mArea)
	{
		$command = "SELECT DISTINCT info.key_name
		    		  FROM {$this->gDBNick}_info info
		   			 WHERE info.$mArea = 1
					   AND info.key_name <> ''
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGoodsComment
	 **/
	function GetGoodsComment($mGoodsID)
	{
		$command = "SELECT id, store_id, date_time, name, email, mess_text, set_voice, ip
		    		  FROM {$this->gDBNick}_store_comment store_comment
		   			 WHERE store_comment.store_id = '$mGoodsID'
				  ORDER BY date_time DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGoodsHitStat
	 **/
	function GetGoodsHitStat($mBuyGoods, $mFromDate, $mDate, $mLimit)
	{
		$table = ( $mBuyGoods ) ? 'ss_store_buy' : 'ss_store';
		$date = ( $mFromDate ) ? " WHERE isday >= '$mDate' " : '';

		$command = "SELECT id, COUNT(id) as how
		    		  FROM {$this->gDBNick}_$table
		   			       $date
		          GROUP BY id
		          ORDER BY how DESC
		             LIMIT $mLimit
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetAllCommentFromDate
	 **/
	function GetAllCommentFromDate($mDateTime)
	{
		$command = "SELECT id, store_id, date_time, name, email, mess_text, set_voice, ip
		    		  FROM {$this->gDBNick}_store_comment store_comment
		   			 WHERE store_comment.date_time >= '$mDateTime'
				  ORDER BY date_time DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * UpdateGoodsVoice
	 **/
	function UpdateGoodsVoice($mGoodsID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_comment
		   			 WHERE store_id = '$mGoodsID'
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);
		$voice_how = $hash[0]*1;

		$command = "SELECT AVG(set_voice)
		    		  FROM {$this->gDBNick}_store_comment
		   			 WHERE store_id = '$mGoodsID'
					   AND set_voice > 0
				  GROUP BY store_id
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);
		$voice_avg = $hash[0]*1;

		$command = "UPDATE {$this->gDBNick}_store
					   SET voice_avg = '$voice_avg',
						   voice_how = '$voice_how'
		   			 WHERE id = '$mGoodsID'
					";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * AddComment
	 **/
	function AddComment($mGoodsID, $mDateTime, $mName, $mEmail, $mMessText, $mSetVoice, $mIP)
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_store_comment store_comment
		   			 WHERE store_comment.store_id = '$mGoodsID'
					   AND store_comment.name = '$mName'
					   AND store_comment.email = '$mEmail'
					   AND store_comment.mess_text = '$mMessText'
					   AND store_comment.set_voice = '$mSetVoice'
					   AND store_comment.ip = '$mIP'
					 LIMIT 1
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		if ( $this->GetNumRows($sth) > 0) return false;

		$command = "SELECT *
		    		  FROM {$this->gDBNick}_comment_block
		   			 WHERE ip = '$mIP'
					 LIMIT 1
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		if ( $this->GetNumRows($sth) > 0) return false;

		$command = "INSERT INTO {$this->gDBNick}_store_comment
		   			 		SET store_id = '$mGoodsID',
								date_time = '$mDateTime',
								name = '$mName',
								email = '$mEmail',
								mess_text = '$mMessText',
								set_voice = '$mSetVoice',
								ip = '$mIP'
					";
		$this->gDB->DoQuery(__LINE__, $command);

		$this->UpdateGoodsVoice($mGoodsID);

		return true;
	}

	/**
	 * RemoveComment
	 **/
	function RemoveComment($mCommentID)
	{
		$command = "SELECT store_id
		    		  FROM {$this->gDBNick}_store_comment
		   			 WHERE id = '$mCommentID'
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		$command = "DELETE
		    		  FROM {$this->gDBNick}_store_comment
		   			 WHERE id = '$mCommentID'
					";
		$this->gDB->DoQuery(__LINE__, $command);

		$this->UpdateGoodsVoice($hash[0]);

		return true;
	}

	/**
	 * UpdateComment
	 **/
	function UpdateComment($mID, $mName, $mEmail, $MessText, $mSetVoice)
	{
		$command = "SELECT store_id
		    		  FROM {$this->gDBNick}_store_comment
		   			 WHERE id = '$mID'
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		$command = "UPDATE {$this->gDBNick}_store_comment
					   SET name = '$mName',
						   email = '$mEmail',
						   mess_text = '$MessText',
						   set_voice = '$mSetVoice'
		   			 WHERE id = '$mID'
					";
		$this->gDB->DoQuery(__LINE__, $command);

		$this->UpdateGoodsVoice($hash[0]);

		return true;
	}

	/**
	 * AddCommentBlock
	 **/
	function AddCommentBlock($mIP, $mDateTime)
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_comment_block
		   			 WHERE ip = '$mIP'
					 LIMIT 1
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		if ( $this->GetNumRows($sth) > 0) return false;

		$command = "INSERT INTO {$this->gDBNick}_comment_block
		   			 		SET ip = '$mIP',
								date_time = '$mDateTime'
					";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetCommentBlock
	 **/
	function GetCommentBlock()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_comment_block
				  ORDER BY date_time DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * RemoveCommentBlock
	 **/
	function RemoveCommentBlock($mIP)
	{
		$command = "DELETE
		    		  FROM {$this->gDBNick}_comment_block
		   			 WHERE ip = '$mIP'
					";

		$this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetFamilyGoods
	 **/
	function GetFamilyGoods($mFamilyID)
	{
		$command = "SELECT store.*
		    		  FROM {$this->gDBNick}_store store
		   			 WHERE store.family = '$mFamilyID'
					   AND store.family > 0
				   	   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY store.price
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCompareGoods
	 **/
	function GetCompareGoods($mGoodsID)
	{
		$command = "SELECT store.*
		    		  FROM {$this->gDBNick}_store store
		   			 WHERE store.id IN ( $mGoodsID )
				   	   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY store.price
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetFamilyInfoValues
	 **/
	function GetFamilyInfoValues($mFamilyID, $mInfoID)
	{
		$command = "SELECT info_value.name, store.id
		    		  FROM {$this->gDBNick}_store store
				 LEFT JOIN {$this->gDBNick}_store_info store_info
				 	    ON store_info.store_id = store.id
					   AND store_info.info_id = '$mInfoID'
				 LEFT JOIN {$this->gDBNick}_info_value info_value
				        ON store_info.value_id = info_value.id
		   			 WHERE store.family = '$mFamilyID'
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY store.price
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCompareInfoValues
	 **/
	function GetCompareInfoValues($mGoodsID, $mInfoID)
	{
		$command = "SELECT info_value.name, store.id
		    		  FROM {$this->gDBNick}_store store
				 LEFT JOIN {$this->gDBNick}_store_info store_info
				 	    ON store_info.store_id = store.id
					   AND store_info.info_id = '$mInfoID'
				 LEFT JOIN {$this->gDBNick}_info_value info_value
				        ON store_info.value_id = info_value.id
		   			 WHERE store.id IN ( $mGoodsID )
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY store.price
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetInfoFamilyGoods
	 **/
	function GetInfoFamilyGoods($mFamilyID)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_family_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_family_tmp
					SELECT store_info.info_id
					  FROM {$this->gDBNick}_store store
				 LEFT JOIN {$this->gDBNick}_store_info store_info
				        ON store_info.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_info info
				 	    ON store_info.info_id = info.id
					 WHERE store.family = '$mFamilyID'
				   	   AND store.kind <> 1
					   AND store.active = 1
					   AND info.in_family = 1
				  GROUP BY store_info.info_id
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT info.id, info.name, info.tlevel, info.key_name, info.isfolder, family.info_id,
						   info.tindex
		    		  FROM {$this->gDBNick}_info info
			     LEFT JOIN {$this->gDBNick}_family_tmp family
				        ON family.info_id = info.id
		   			 WHERE info.in_family = 1
					   AND (info.isfolder = 1 OR family.info_id IS NOT NULL)
				  ORDER BY info.absindex DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetInfoCompareGoods
	 **/
	function GetInfoCompareGoods($mGoodsID)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_compare_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_compare_tmp
					SELECT store_info.info_id
					  FROM {$this->gDBNick}_store store
				 LEFT JOIN {$this->gDBNick}_store_info store_info
				        ON store_info.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_info info
				 	    ON store_info.info_id = info.id
					 WHERE store.id IN ( $mGoodsID )
				   	   AND store.kind <> 1
					   AND store.active = 1
					   AND info.in_preview = 1
				  GROUP BY store_info.info_id
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT info.id, info.name, info.tlevel, info.key_name, info.isfolder, compare.info_id,
						   info.tindex
		    		  FROM {$this->gDBNick}_info info
			     LEFT JOIN {$this->gDBNick}_compare_tmp compare
				        ON compare.info_id = info.id
		   			 WHERE info.in_preview = 1
					   AND (info.isfolder = 1 OR compare.info_id IS NOT NULL)
				  ORDER BY info.absindex DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * CalcBaseParamGoods
	 **/
	function CalcBaseParamGoods($mGoodsID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_param param,
				           {$this->gDBNick}_store_param store_param
		   			 WHERE store_param.param_id = param.id
					   AND store_param.store_id = '$mGoodsID'
					   AND param.isfolder = 0
					   AND param.phis_type = 0
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * UpdateGoodsCache
	 **/
	function UpdateGoodsCache($mGoodsID, $mKey, $mValue)
	{
		$command = "UPDATE {$this->gDBNick}_store
					   SET $mKey = '$mValue'
		   			 WHERE id = '$mGoodsID'
					";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetParamGoods
	 **/
	function GetParamGoods($mGoodsID)
	{
		$command = "SELECT param.id, param.name, param.get_type, param.dir_name, param.col_num,
						   param.xpos, param.ypos, param.w, param.h, param.phis_type
		    		  FROM {$this->gDBNick}_param param,
				           {$this->gDBNick}_store_param store_param
		   			 WHERE store_param.param_id = param.id
					   AND store_param.store_id = '$mGoodsID'
					   AND param.isfolder = 0
				  ORDER BY param.absindex
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}


	/**
	 * GetParam
	 **/
	function GetParam($mParamID)
	{
		$command = "SELECT param.id, param.name, param.get_type, param.dir_name, param.col_num,
						   param.xpos, param.ypos, param.w, param.h, param.phis_type
		    		  FROM {$this->gDBNick}_param param
		   			 WHERE param.id = '$mParamID'
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}


	/**
	 * GoodsPersonalValue
	 **/
	function GoodsPersonalValue($mStoreID, $mParamValueID)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_store_add
			   		 WHERE store_id = '$mStoreID'
					   AND param_value_id = '$mParamValueID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return ( $this->GetNumRows($sth) > 0 );
	}

	/**
	 * GetParamGoodsValues
	 **/
	function GetParamGoodsValues($mGoodsID, $mParamID, $mDefaultHow)
	{
		$command = "SELECT param_value.id, param_value.name, param_value.descr, param_value.proc, param_value.suma,
						   store_add.price, store_add.code, store_add.status,
						   IF (store_add.price IS NOT NULL, store_add.how, $mDefaultHow)
		    		  FROM {$this->gDBNick}_param_value param_value
				 LEFT JOIN {$this->gDBNick}_store_add store_add
				        ON param_value.id = store_add.param_value_id
					   AND store_add.store_id = '$mGoodsID'
		   			 WHERE param_value.param_id = '$mParamID'
		               AND ( store_add.status IS NULL OR store_add.status < 4 )
				  ORDER BY param_value.pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}


	/**
	 * GetGoodsParamValue
	 **/
	function GetGoodsParamValue($mGoodsID, $mParamValueID)
	{
		$command = "SELECT param.name AS param_name, param_value.name, param_value.proc, param_value.suma,
						   param_value.tax AS param_tax,
						   store_add.price, store_add.code, store_add.status, store_add.tax
		    		  FROM (
						   {$this->gDBNick}_param_value param_value,
					  	   {$this->gDBNick}_param param
						   )
				 LEFT JOIN {$this->gDBNick}_store_add store_add
				        ON param_value.id = store_add.param_value_id
					   AND store_add.store_id = '$mGoodsID'
		   			 WHERE param_value.param_id = param.id
					   AND param_value.id = '$mParamValueID'
					   AND ( store_add.status IS NULL OR store_add.status < 4 )
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetGoodsDiscParam
	 **/
	function GetGoodsDiscParam($mGoodsID, $mHow, $mCategoryID = 0)
	{
		$command = "SELECT SUM(discount.proc), SUM(discount.suma)
		    		  FROM {$this->gDBNick}_store_disc store_disc,
				           {$this->gDBNick}_discount discount
		   			 WHERE store_disc.disc_id = discount.id
					   AND store_disc.store_id = '$mGoodsID'
					   AND discount.from_how <= '$mHow'
					   AND discount.category_id = $mCategoryID
					   AND discount.isfolder = 0
		          GROUP BY discount.from_how
			      ORDER BY discount.from_how DESC
				     LIMIT 1
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return array($hash[0], $hash[1]);
	}

	/**
	 * GetGoodsDiscounts
	 **/
	function GetGoodsDiscounts($mGoodsID, $mCategoryID)
	{
		$command = "SELECT discount.id, discount.name, discount.descr, discount.tlevel, discount.tindex, discount.absindex,
						   discount.from_how, discount.category_id , discount.proc, discount.suma, discount.isfolder,
						   store_disc.store_id, category.name
		    		  FROM {$this->gDBNick}_discount discount
			     LEFT JOIN {$this->gDBNick}_store_disc store_disc
				        ON store_disc.disc_id = discount.id AND store_disc.store_id = '$mGoodsID'
		         LEFT JOIN {$this->gDBNick}_category category
		 			    ON category.id = discount.category_id
					 WHERE discount.isfolder = 1 OR ( store_disc.store_id IS NOT NULL AND discount.category_id = $mCategoryID )
				  ORDER BY discount.absindex DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNeedInfo
	 **/
	function GetNeedInfo($mGoodsID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_info info,
			     		   {$this->gDBNick}_store_info store_info
		   			 WHERE store_info.info_id = info.id
					   AND store_info.store_id = '$mGoodsID'
					   AND info.in_main = 0
					   AND info.in_preview = 1
					   AND info.isfolder = 0
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetNeedDiscount
	 **/
	function GetNeedDiscount($mGoodsID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_disc
		   			 WHERE store_id = '$mGoodsID'
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetNeedAdvert
	 **/
	function GetNeedAdvert($mGoodsID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_adv
		   			 WHERE store_id = '$mGoodsID'
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetNeedFamily
	 **/
	function GetNeedFamily($mFamilyID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store store
		   			 WHERE family = '$mFamilyID'
					   AND store.kind <> 1
					   AND active = 1
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}


	/**
	 * GetNeedParam
	 **/
	function GetNeedParam($mGoodsID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_param
		   			 WHERE store_id = '$mGoodsID'
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetFilterGoods
	 **/
	function GetFilterGoods()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_filters
	   				 WHERE in_firstpage = 0
			      ORDER BY pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetFilterIndex
	 **/
	function GetFilterIndex()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_filters
	   				 WHERE in_firstpage = 1
			      ORDER BY pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetFilterValuesGoods
	 **/
	function GetFilterValuesGoods($mInfoID)
	{
		$command = "SELECT info_value.id, info_value.name
		    		  FROM {$this->gDBNick}_store_info store_info,
						   {$this->gDBNick}_info_value info_value,
						   {$this->gDBNick}_store_tmp store
		   			 WHERE store_info.info_id = '$mInfoID'
					   AND store_info.store_id = store.id
					   AND store_info.value_id = info_value.id
				  GROUP BY info_value.id
				  ORDER BY info_value.pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetFilterValuesIndex
	 **/
	function GetFilterValuesIndex($mInfoID)
	{
		$command = "SELECT DISTINCT info_value.id, info_value.name
		    		  FROM {$this->gDBNick}_info_value info_value,
					       {$this->gDBNick}_store_info store_info,
						   {$this->gDBNick}_store store
		   			 WHERE info_value.info_id = '$mInfoID'
					   AND info_value.id = store_info.value_id
					   AND store_info.store_id = store.id
					   AND store.kind <> 1
					   AND store.active = 1
			      ORDER BY info_value.pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNeedTopicFilter
	 **/
	function GetNeedTopicFilter($mTopicID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_topic_filter
		   			 WHERE topic_id = '$mTopicID'
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0] > 0;
	}

	/**
	 * GetInfoTopicFilter
	 **/
	function GetInfoTopicFilter($mTopicID)
	{
		$command = "SELECT info.id, info.name, info.tlevel, info.tindex, info.isfolder, topic_filter.topic_id,
						   topic_filter.values_col, topic_filter.list_type
		    		  FROM {$this->gDBNick}_info info
			     LEFT JOIN {$this->gDBNick}_topic_filter topic_filter
				        ON topic_filter.info_id = info.id
		 			   AND topic_filter.topic_id = '$mTopicID'
		   			 WHERE ( info.isfolder = 1  AND ( info.in_preview = 1 OR info.in_main = 1 OR info.in_family = 1 ) )
					    OR topic_filter.topic_id IS NOT NULL
				  ORDER BY info.absindex DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetTopicFilterValues
	 **/
	function GetTopicFilterValues($mInfoID, $mTopicID)
	{
		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_topic_filter_area topic_filter_area
					 WHERE topic_filter_area.topic_id = '$mTopicID'
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		if ( $hash[0] > 0 )
		{
			$command = "SELECT DISTINCT info_value.id, info_value.name
			    		  FROM {$this->gDBNick}_info_value info_value STRAIGHT_JOIN
							   {$this->gDBNick}_topic_filter topic_filter,
							   {$this->gDBNick}_topic_filter_area topic_filter_area STRAIGHT_JOIN
							   {$this->gDBNick}_topic_store topic_store STRAIGHT_JOIN
							   {$this->gDBNick}_store_info store_info,
							   {$this->gDBNick}_store store
			   			 WHERE info_value.info_id = '$mInfoID'
						   AND info_value.info_id = topic_filter.info_id
						   AND info_value.id = store_info.value_id
						   AND store_info.store_id = store.id
						   AND topic_store.store_id = store.id
						   AND topic_filter_area.area_topic_id = topic_store.topic_id
						   AND topic_filter_area.topic_id = '$mTopicID'
						   AND store.kind <> 1
						   AND store.active = 1
				      ORDER BY info_value.pos
						";
		}
		else
		{
			$command = "SELECT DISTINCT info_value.id, info_value.name
			    		  FROM {$this->gDBNick}_info_value info_value,
							   {$this->gDBNick}_topic_filter topic_filter,
							   {$this->gDBNick}_store_info store_info,
							   {$this->gDBNick}_store store
			   			 WHERE info_value.info_id = '$mInfoID'
						   AND info_value.info_id = topic_filter.info_id
						   AND info_value.id = store_info.value_id
						   AND store_info.store_id = store.id
						   AND store.kind <> 1
						   AND store.active = 1
				      ORDER BY info_value.pos
						";
		}

		return $this->gDB->DoQuery(__LINE__, $command);
	}



	//***************************************************************************************************
	//
	// 									   Goods configurator
	//
	//***************************************************************************************************

	/**
	 * GetConfTree
	 **/
	function GetConfTree()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_conf
				  ORDER BY absindex
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetConfig
	 **/
	function GetConfig($mConfigID)
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_conf
					 WHERE id = '$mConfigID'
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetConfUnit
	 **/
	function GetConfUnit($mConfigID)
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_conf_unit
					 WHERE conf_id = '$mConfigID'
				  ORDER BY pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetConfGoods
	 **/
	function GetConfGoods($mConfUnitID, $mSortByPrice)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_conf_store_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_conf_store_tmp
					SELECT DISTINCT store.id
		    		  FROM {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_conf_store conf_store
		   			 WHERE conf_store.store_id = store.id
		     		   AND conf_store.conf_unit_id = '$mConfUnitID'
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY conf_store.pos
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_conf_store_tmp
					SELECT DISTINCT store.id
		    		  FROM {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_topic_store topic_store,
						   {$this->gDBNick}_conf_topic conf_topic
		   			 WHERE topic_store.store_id = store.id
					   AND topic_store.topic_id = conf_topic.topic_id
		     		   AND conf_topic.conf_unit_id = '$mConfUnitID'
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY conf_topic.pos, topic_store.pos
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$order = '';
		if ( $mSortByPrice )
		{
			$order = ' ORDER BY store.price ASC';
		}
		$command = "SELECT DISTINCT store.id, store.code, store.meas, store.name, store.status, store.price
		    		  FROM {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_conf_store_tmp conf_store_tmp
		   			 WHERE conf_store_tmp.id = store.id
					       $order
					";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}


	/**
	 * TestConfComp
	 **/
	function TestConfComp($mGoodsID)
	{
		$command = "SELECT store_comp.inc_store_id, store_comp.how
		    		  FROM {$this->gDBNick}_store_comp store_comp,
					  	   {$this->gDBNick}_conf_store_tmp conf_store_tmp
		   			 WHERE conf_store_tmp.id = store_comp.inc_store_id
					   AND store_comp.base_store_id = '$mGoodsID'
				     LIMIT 1
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchRow($sth);
	}


	/**
	 * GetConfGoodsList
	 **/
	function GetConfGoodsList($mGoodsList)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_conf_store_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_conf_store_tmp
					SELECT conf_store.store_id, conf_store.conf_unit_id
		    		  FROM {$this->gDBNick}_conf_store conf_store
		   			 WHERE conf_store.store_id IN ( $mGoodsList )
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_conf_store_tmp
					SELECT topic_store.store_id, conf_topic.conf_unit_id
		    		  FROM {$this->gDBNick}_topic_store topic_store,
						   {$this->gDBNick}_conf_topic conf_topic
		   			 WHERE topic_store.topic_id = conf_topic.topic_id
					   AND topic_store.store_id IN ( $mGoodsList )
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT store.id, store.code, store.meas, store.name, store.status, store.descr,
						   store.price, store.part_proc, store.step_order, store.min_order, store.tax,
						   store.disc_price, store.active, store.apendix_type, store.how, store.xpos,
						   store.ypos, store.w, store.h, store.kind, store.family, store_new.store_id,
						   store_hits.store_id, store.how, store.seo_link, store.seo_alt
		    		  FROM (
						   {$this->gDBNick}_conf_store_tmp conf_store_tmp,
						   {$this->gDBNick}_conf_unit conf_unit,
						   {$this->gDBNick}_store store
						   )
				 LEFT JOIN {$this->gDBNick}_store_new store_new
				        ON store_new.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_store_hits store_hits
				        ON store_hits.store_id = store.id
		   			 WHERE store.id = conf_store_tmp.store_id
					   AND conf_store_tmp.conf_unit_id = conf_unit.id
				  GROUP BY store.id
				  ORDER BY conf_unit.pos
					";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}



	//***************************************************************************************************
	//
	// 									   		Goods
	//
	//***************************************************************************************************

	/**
	 * GetGoods
	 **/
	function GetGoods($mGoodsID)
	{
		$command = "SELECT store.id, store.code, store.meas, store.name, store.status, store.descr,
						   store.price, store.part_proc, store.step_order, store.min_order, store.tax,
						   store.disc_price, store.active, store.apendix_type, store.how, store.xpos,
						   store.ypos, store.w, store.h, store.kind, store.family,
						   store_new.store_id AS is_new, store_hits.store_id AS is_hit, store.how AS max_order,
						   store.seo_title, store.seo_descr, store.seo_keywords, store.seo_link, store.seo_alt,
						   store.conf_id, store.seo_psu, store.provider_id, store.prov_price
		    		  FROM {$this->gDBNick}_store store
			 	 LEFT JOIN {$this->gDBNick}_store_new store_new
				        ON store_new.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_store_hits store_hits
				        ON store_hits.store_id = store.id
		   			 WHERE id = '$mGoodsID'
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * FindGoodsPSU
	 **/
	function FindGoodsPSU($mPSU)
	{
		$command = "SELECT id
	    	  	  	  FROM {$this->gDBNick}_store
				     WHERE seo_psu = '$mPSU'
			       ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0]*1;
	}

	/**
	 * GetGoodsForXML
	 **/
	function GetGoodsForXML($mTopic)
	{
		$command = "SELECT store.id, store.code, store.meas, store.name, store.status, store.descr,
						   store.price, store.part_proc, store.step_order, store.min_order, store.tax,
						   store.disc_price, store.active, store.kind, store.how, store.seo_psu
		    		  FROM {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_topic_store topic_store
		   			 WHERE topic_store.store_id = store.id
		     		   AND topic_store.topic_id = '$mTopic'
					   AND store.kind <> 1
					   AND store.active = 1
					   AND store.in_xml = 1
				  ORDER BY topic_store.pos
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * MakeDirGoods
	 **/
	function MakeDirGoods($mTopic, $mStatus0, $mStatus1, $mStatus2, $mStatus3)
	{
		$status = '';
		if ( $mStatus0 ) $status .= ' store.status = 0 OR ';
		if ( $mStatus1 ) $status .= ' store.status = 1 OR ';
		if ( $mStatus2 ) $status .= ' store.status = 2 OR ';
		if ( $mStatus3 ) $status .= ' store.status = 3 OR ';
		if ( strlen($status) > 0 ) $where .= ' AND ('.substr($status, 0, -3).') ';

		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_store_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_store_tmp
					SELECT store.id
		    		  FROM {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_topic_store topic_store
		   			 WHERE topic_store.store_id = store.id
		     		   AND topic_store.topic_id = '$mTopic'
					   AND store.kind <> 1
					   AND store.active = 1
		                   $where
				  ORDER BY topic_store.pos
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_tmp
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * MakeSelectGoods
	 **/
	function MakeSelectGoods($mTopicID, $mPriceFrom, $mPriceTo, $mStatus0, $mStatus1, $mStatus2, $mStatus3)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_store_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_topic_filter_area topic_filter_area
					 WHERE topic_filter_area.topic_id = '$mTopicID'
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		if ( $hash[0] > 0 )
		{
			$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_store_tmp
						SELECT DISTINCT store.id
			    		  FROM {$this->gDBNick}_store store,
						  	   {$this->gDBNick}_store_info store_info,
							   {$this->gDBNick}_topic_filter topic_filter,
							   {$this->gDBNick}_topic_store topic_store,
							   {$this->gDBNick}_topic_filter_area topic_filter_area
		   				 WHERE topic_filter.topic_id = '$mTopicID'
		     		   	   AND topic_filter.info_id = store_info.info_id
						   AND store_info.store_id = store.id
						   AND topic_filter_area.topic_id = '$mTopicID'
						   AND topic_filter_area.area_topic_id = topic_store.topic_id
						   AND topic_store.store_id = store.id
						   AND store.kind <> 1
						   AND store.active = 1
					   ";
		}
		else
		{
			$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_store_tmp
						SELECT DISTINCT store.id
			    		  FROM {$this->gDBNick}_store store,
						  	   {$this->gDBNick}_store_info store_info,
							   {$this->gDBNick}_topic_filter topic_filter
		   				 WHERE topic_filter.topic_id = '$mTopicID'
		     		   	   AND topic_filter.info_id = store_info.info_id
						   AND store_info.store_id = store.id
						   AND store.kind <> 1
						   AND store.active = 1
					   ";
		}

		if ( $mPriceFrom > 0 ) $command .= ' AND store.price >= \''.$mPriceFrom.'\' ';
		if ( $mPriceTo > 0 ) $command .= ' AND store.price <= \''.$mPriceTo.'\' ';

		$status = '';
		if ( $mStatus0 ) $status .= ' store.status = 0 OR ';
		if ( $mStatus1 ) $status .= ' store.status = 1 OR ';
		if ( $mStatus2 ) $status .= ' store.status = 2 OR ';
		if ( $mStatus3 ) $status .= ' store.status = 3 OR ';
		if ( strlen($status) > 0 ) $command .= ' AND ('.substr($status, 0, -3).') ';

		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_tmp
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * MakeSearchGoods
	 **/
	function MakeSearchGoods($mWords, $mStatus0, $mStatus1, $mStatus2, $mStatus3)
	{
    	$search = 'AND (';
   		$words = explode(' ', $mWords);
		foreach ( $words as $word )
		{
			$query = "((store.name LIKE '%$word%')";
			$query .= " OR (store.code LIKE '%$word%')";
        	$query .= " OR (store.descr LIKE '%$word%')";
		    $query .= " OR (info_value.name LIKE '%$word%')";
	    	$query .= ')';
        	$search .= ( $search == 'AND (' ) ? $query : " AND $query";
    	}
		$search .= ')';
		if ( $search == 'AND ()' || $search == 'AND ())')
		{
			$search = '';
		}

		$status = '';
		if ( $mStatus0 ) $status .= ' store.status = 0 OR ';
		if ( $mStatus1 ) $status .= ' store.status = 1 OR ';
		if ( $mStatus2 ) $status .= ' store.status = 2 OR ';
		if ( $mStatus3 ) $status .= ' store.status = 3 OR ';
		if ( strlen($status) > 0 ) $search .= ' AND ('.substr($status, 0, -3).') ';

		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_store_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_store_tmp
					SELECT DISTINCT store.id
		    		  FROM {$this->gDBNick}_store store
			     LEFT JOIN {$this->gDBNick}_store_info store_info
				        ON store_info.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_info_value info_value
				 		ON info_value.id = store_info.value_id
		   			 WHERE store.kind <> 1
					   AND store.active = 1
					       $search
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_tmp
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}


	/**
	 * MakeExpSearchGoods
	 **/
	function MakeExpSearchGoods($mWords, $mTopic, $mStatus0, $mStatus1, $mStatus2, $mStatus3, $mNew, $mHit, $mDisc, $mPriceFrom, $mPriceTo )
	{

		$from = '';
		$where = '';

		if ( $mTopic > 0 )
		{
			$command = "DROP TABLE IF EXISTS {$this->gDBNick}_topic_tmp";
			$this->gDB->DoQuery(__LINE__, $command);

			$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_topic_tmp
						SELECT id, tlevel
		    			FROM {$this->gDBNick}_topic topic
			   		    WHERE active = 1
						  AND istype < 3
						  AND id = '$mTopic'
					   ";
			$this->gDB->DoQuery(__LINE__, $command);

			$command = "SELECT tlevel FROM {$this->gDBNick}_topic_tmp";
			$sth = $this->gDB->DoQuery(__LINE__, $command);
			$hash = $this->FetchRow($sth);
			$level = $hash[0];

			while ( true )
			{
				$command = "SELECT topic.id, topic.tlevel
		    	  	  		  FROM {$this->gDBNick}_topic topic,
								   {$this->gDBNick}_topic_tmp topic_tmp
				             WHERE topic.tindex = topic_tmp.id
							   AND topic.tlevel > $level
				           ";
				$sth = $this->gDB->DoQuery(__LINE__, $command);
				$rows = $this->GetNumRows($sth);
				if ( $rows == 0 ) break;
				for ( $i = 1; $i <= $rows; $i++ )
				{
					$hash = $this->FetchRow($sth);
					$command = "INSERT INTO {$this->gDBNick}_topic_tmp SET id = $hash[0], tlevel = $hash[1]";
					$this->gDB->DoQuery(__LINE__, $command);
				}
				$level++;
			}
			$from = ", {$this->gDBNick}_topic_tmp topic_tmp, {$this->gDBNick}_topic_store topic_store ";
			$where = ' AND topic_store.store_id = store.id AND topic_store.topic_id = topic_tmp.id ';
		}

		$status = '';
		if ( $mStatus0 ) $status .= ' store.status = 0 OR ';
		if ( $mStatus1 ) $status .= ' store.status = 1 OR ';
		if ( $mStatus2 ) $status .= ' store.status = 2 OR ';
		if ( $mStatus3 ) $status .= ' store.status = 3 OR ';
		if ( strlen($status) > 0 ) $where .= ' AND ('.substr($status, 0, -3).') ';

		if ( $mNew )
		{
			$from .= ", {$this->gDBNick}_store_new store_new ";
			$where .= ' AND store_new.store_id = store.id ';
		}
		if ( $mHit )
		{
			$from .= ", {$this->gDBNick}_store_hits store_hits ";
			$where .= ' AND store_hits.store_id = store.id ';
		}
		if ( $mDisc )
		{
			$where .= ' AND store.disc_price > 0 ';
		}
		if ( $mPriceFrom )
		{
			$where .= " AND IF(store.disc_price > 0, store.disc_price, store.price) >= '$mPriceFrom' ";
		}
		if ( $mPriceTo )
		{
			$where .= " AND IF(store.disc_price > 0, store.disc_price, store.price) <= '$mPriceTo' ";
		}

		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_limit_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_limit_tmp
					SELECT DISTINCT store.id
		   			  FROM {$this->gDBNick}_store store
		                   $from
				     WHERE store.kind <> 1
					   AND store.active = 1
						   $where
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$search = 'AND (';
   		$words = explode(' ', $mWords);
		foreach ( $words as $word )
		{
			$query = "((store.name LIKE '%$word%')";
			$query .= " OR (store.code LIKE '%$word%')";
        	$query .= " OR (store.descr LIKE '%$word%')";
		    $query .= " OR (info_value.name LIKE '%$word%')";
	    	$query .= ')';
        	$search .= ( $search == 'AND (' ) ? $query : " AND $query";
    	}
		$search .= ')';
		if ( $search == 'AND ()' || $search == 'AND ())')
		{
			$search = '';
		}

		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_store_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_store_tmp
					SELECT DISTINCT store.id
		    		  FROM (
						   {$this->gDBNick}_store store,
						   {$this->gDBNick}_limit_tmp limit_tmp
						   )
			     LEFT JOIN {$this->gDBNick}_store_info store_info
				        ON store_info.store_id = limit_tmp.id
				 LEFT JOIN {$this->gDBNick}_info_value info_value
				 		ON info_value.id = store_info.value_id
		   			 WHERE store.id = limit_tmp.id
					       $search
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_tmp
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}


	/**
	 * ApplyFilterGoods
	 **/
	function ApplyFilterGoods($mVars)
	{
		$tables = '';
		$join = '';
		$filters = explode('and', $mVars['filter']);
		foreach ($filters as $id)
		{
			$id *= 1;
			if ( $id == 0 ) continue;
			if ( !isset($mVars['filter'.$id]) ) continue;
			$tables .= ' STRAIGHT_JOIN '.$this->gDBNick.'_store_info store_info'.$id;
			$join .= "AND store_info{$id}.store_id = store.id
			          AND store_info{$id}.value_id
					   IN (";
			foreach ( explode('or', $mVars['filter'.$id]) as $value_id )
			{
				$value_id *= 1;
				if ( $value_id > 0 ) $join .= $value_id.',';
			}
			$join = rtrim($join, ',').') ';
		}

		$join = ltrim($join, 'AND');

		if ( strlen($join) == 0 ) $join = ' 1 < 0 ';

		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_store_tmp_filter";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_store_tmp_filter
					SELECT DISTINCT store.id
		    		  FROM {$this->gDBNick}_store_tmp store
						   $tables
		   			 WHERE $join
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT COUNT(*)
		    		  FROM {$this->gDBNick}_store_tmp_filter
					";

		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetMainGoods
	 **/
	function GetMainGoods($mApplyFilter, $mOrder, $mOrderByVoice = false)
	{
		// is use filter?
		$table = ( $mApplyFilter ) ? 'store_tmp_filter' : 'store_tmp';

		switch ( $mOrder )
		{
			case 'price' :
				$order = 'ORDER BY store.price';
				break;
			case 'price_desc' :
				$order = 'ORDER BY store.price DESC';
				break;
			case 'name' :
				$order = 'ORDER BY store.name';
				break;
			case 'name_desc' :
				$order = 'ORDER BY store.name DESC';
				break;
			default:
			$order = ( $mOrderByVoice ) ? 'ORDER BY voice_avg DESC' : '';
		}

		$command = "SELECT store.id, store.code, store.meas, store.name, store.status, store.descr,
						   store.price, store.part_proc, store.step_order, store.min_order, store.tax,
						   store.disc_price, store.active, store.apendix_type, store.how, store.xpos,
						   store.ypos, store.w, store.h, store.kind, store.family,
						   store_new.store_id, store_hits.store_id, store.how,
						   store.seo_link, store.seo_alt, store.voice_avg, store.voice_how, store.seo_psu,
						   store.cache_base_param, store.cache_info_preview, store.cache_family,
						   store.cache_param, store.cache_disc, store.cache_advert
		    		  FROM (
						   {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_$table store_tmp
						   )
				 LEFT JOIN {$this->gDBNick}_store_new store_new
				        ON store_new.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_store_hits store_hits
				        ON store_hits.store_id = store.id
		   			 WHERE store_tmp.id = store.id
					       $order
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		$command = "DROP TABLE {$this->gDBNick}_$table";
		$this->gDB->DoQuery(__LINE__, $command);

		return $sth;
	}

	/**
	 * GetAdvertForTopic
	 **/
	function GetAdvertForTopic($mTopicID)
	{
		$command = "SELECT advert.id, topic_adv.new_name, advert.descr, topic_adv.dir_name, topic_adv.place_code, topic_adv.col_num,
						   topic_adv.iscompare, topic_adv.one_time
		    		  FROM {$this->gDBNick}_advert advert,
					  	   {$this->gDBNick}_topic_adv topic_adv
		   			 WHERE topic_adv.adv_id = advert.id
					   AND topic_adv.topic_id = '$mTopicID'
		     		   AND advert.isfolder = 0
					   AND ( topic_adv.place_type = 0 OR topic_adv.place_type = 2 )
				  ORDER BY topic_adv.pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetAdvertForGoods
	 **/
	function GetAdvertForGoods($mTopicID, $mGoodsID)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_advert_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_advert_tmp
					SELECT advert.id, topic_adv.new_name, advert.descr, topic_adv.dir_name, topic_adv.place_code, topic_adv.col_num, topic_adv.iscompare
		    		  FROM {$this->gDBNick}_advert advert,
					  	   {$this->gDBNick}_topic_adv topic_adv
		   			 WHERE topic_adv.adv_id = advert.id
					   AND topic_adv.topic_id = '$mTopicID'
		     		   AND advert.isfolder = 0
					   AND ( topic_adv.place_type = 1 OR topic_adv.place_type = 2 )
				  ORDER BY topic_adv.pos
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_advert_tmp
					SELECT advert.id, store_adv.new_name, advert.descr, store_adv.dir_name, store_adv.place_code, store_adv.col_num, store_adv.iscompare
		    		  FROM {$this->gDBNick}_advert advert,
					  	   {$this->gDBNick}_store_adv store_adv
		   			 WHERE store_adv.adv_id = advert.id
					   AND store_adv.store_id = '$mGoodsID'
		     		   AND advert.isfolder = 0
				  ORDER BY store_adv.pos
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT id, new_name, descr, dir_name, place_code, col_num, iscompare
		    		  FROM {$this->gDBNick}_advert_tmp
		          GROUP BY id
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetAdvertGoods
	 **/
	function GetAdvertGoods($mAdvertID)
	{
		$command = "SELECT store.id, store.code, store.name, store.descr, store.price, store.disc_price,
						   store_new.store_id, store_hits.store_id,
						   store.status, store.xpos, store.ypos, store.w, store.h,
						   store.seo_link, store.seo_alt, store.seo_psu, store.cache_base_param
		    		  FROM (
						   {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_adv_store adv_store
						   )
				 LEFT JOIN {$this->gDBNick}_store_new store_new
				        ON store_new.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_store_hits store_hits
				        ON store_hits.store_id = store.id
		   			 WHERE adv_store.store_id = store.id
		     		   AND adv_store.adv_id = '$mAdvertID'
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY adv_store.pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetInfoAdvertGoods
	 **/
	function GetInfoAdvertGoods($mAdvertID)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_advert_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_advert_tmp
					SELECT store_info.info_id
					  FROM (
						   {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_adv_store adv_store
						   )
				 LEFT JOIN {$this->gDBNick}_store_info store_info
				        ON store_info.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_info info
				 	    ON store_info.info_id = info.id
					 WHERE adv_store.store_id = store.id
		     		   AND adv_store.adv_id = '$mAdvertID'
					   AND store.kind <> 1
					   AND store.active = 1
				  GROUP BY store_info.info_id
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT info.id, info.name, info.tlevel, info.key_name, info.isfolder, advert_tmp.info_id,
						   info.tindex
		    		  FROM {$this->gDBNick}_info info
			     LEFT JOIN {$this->gDBNick}_advert_tmp advert_tmp
				        ON advert_tmp.info_id = info.id
		   			 WHERE info.in_family = 1
					   AND (info.isfolder = 1 OR advert_tmp.info_id IS NOT NULL)
				  ORDER BY info.absindex DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetAdvertInfoValues
	 **/
	function GetAdvertInfoValues($mAdvertID, $mInfoID)
	{
		$command = "SELECT info_value.name, store.id
		    		  FROM (
						   {$this->gDBNick}_store store,
						   {$this->gDBNick}_adv_store adv_store
						   )
				 LEFT JOIN {$this->gDBNick}_store_info store_info
				 	    ON store_info.store_id = store.id
					   AND store_info.info_id = '$mInfoID'
				 LEFT JOIN {$this->gDBNick}_info_value info_value
				        ON store_info.value_id = info_value.id
		   			 WHERE adv_store.store_id = store.id
		     		   AND adv_store.adv_id = '$mAdvertID'
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY adv_store.pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCompGoods
	 **/
	function GetCompGoods($mStoreID)
	{
		$command = "SELECT store.id, store.code, store.name, store.price, store.disc_price, store.kind,
						   store_comp.how, store.seo_link, store.seo_alt, store.seo_psu
		    		  FROM {$this->gDBNick}_store store,
				           {$this->gDBNick}_store_comp store_comp
		   			 WHERE store_comp.inc_store_id = store.id
		     		   AND store_comp.base_store_id = '$mStoreID'
					   AND store.active = 1
				  ORDER BY store_comp.pos
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNewGoods
	 **/
	function GetNewGoods($mHow)
	{
		$command = "SELECT store.id, store.code, store.name, store.descr, store.price, store.disc_price,
						   store_hits.store_id, store.status, store.seo_link, store.seo_alt, store.seo_psu, store.cache_base_param
		    		  FROM (
						   {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_store_new store_new
						   )
				 LEFT JOIN {$this->gDBNick}_store_hits store_hits
				        ON store_hits.store_id = store.id
		   			 WHERE store_new.store_id = store.id
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY store_new.pos
				     LIMIT 0, $mHow
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNewTopicGoods
	 **/
	function GetNewTopicGoods($mTopic, $mHow)
	{
		$command = "SELECT store.id, store.code, store.name, store.descr, store.price, store.disc_price,
						   store_hits.store_id, store.status, store.seo_link, store.seo_alt, store.seo_psu, store.cache_base_param
		    		  FROM (
						   {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_store_new store_new,
						   {$this->gDBNick}_topic_store topic_store
						   )
				 LEFT JOIN {$this->gDBNick}_store_hits store_hits
				        ON store_hits.store_id = store.id
		   			 WHERE store_new.store_id = store.id
					   AND topic_store.store_id = store.id
					   AND topic_store.topic_id = '$mTopic'
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY store_new.pos
				     LIMIT 0, $mHow
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetHitGoods
	 **/
	function GetHitGoods($mHow)
	{
		$command = "SELECT store.id, store.code, store.name, store.descr, store.price, store.disc_price,
						   store_new.store_id, store.status, store.seo_link, store.seo_alt, store.seo_psu, store.cache_base_param
		    		  FROM (
						   {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_store_hits store_hits
						   )
				 LEFT JOIN {$this->gDBNick}_store_new store_new
				        ON store_new.store_id = store.id
		   			 WHERE store_hits.store_id = store.id
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY RAND()
				     LIMIT 0, $mHow
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGiftGoods
	 **/
	function GetGiftGoods()
	{
		$command = "SELECT store.id, store.code, store.name, store.descr, store.price, store.disc_price,
						   store_gift.start_sum, store_gift.end_sum, store.seo_link, store.seo_alt, store.seo_psu
		    		  FROM {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_store_gift store_gift
		   			 WHERE store_gift.store_id = store.id
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY store_gift.start_sum, store_gift.end_sum
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGiftForSum
	 **/
	function GetGiftForSum($mSuma)
	{
		$command = "SELECT store.id, store.code, store.name, store.descr, store.provider_id, store.prov_price, store.tax
		    		  FROM {$this->gDBNick}_store store,
					  	   {$this->gDBNick}_store_gift store_gift
		   			 WHERE store_gift.store_id = store.id
					   AND store.kind <> 1
					   AND store.active = 1
					   AND ((store_gift.start_sum <= $mSuma) AND ((store_gift.end_sum >= $mSuma) OR (store_gift.end_sum = 0)))
				  ORDER BY store_gift.start_sum, store_gift.end_sum
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetDiscountGoods
	 **/
	function GetDiscountGoods($mHow)
	{
		$command = "SELECT store.id, store.code, store.name, store.descr, store.price, store.disc_price,
						   store_new.store_id, store_hits.store_id, store.status, store.seo_link, store.seo_alt, store.seo_psu, store.cache_base_param
		    		  FROM {$this->gDBNick}_store store
				 LEFT JOIN {$this->gDBNick}_store_new store_new
				        ON store_new.store_id = store.id
				 LEFT JOIN {$this->gDBNick}_store_hits store_hits
				        ON store_hits.store_id = store.id
		   			 WHERE store.disc_price > 0
					   AND store.kind <> 1
					   AND store.active = 1
				  ORDER BY RAND()
				     LIMIT $mHow
				  ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	//***************************************************************************************************
	//
	// 									   		Sections
	//
	//***************************************************************************************************

	/**
	 * GetPath
	 **/
	function GetPath($mCurrTopicID)
	{
		$command = "SELECT tindex, tlevel, id, name, in_cat_down, in_cat_center, in_cat_menu, in_cat_left, link_title, istype, link, seo_psu
		    	  	  FROM {$this->gDBNick}_topic
					 WHERE id = '$mCurrTopicID'
					 AND active > 0
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		$path = array();
		$link_title = array();
		$link = array();
		if ($hash[4] == 1 || $hash[5] == 1 || $hash[6] == 1 || $hash[7] == 1 || true )
		{
			$path[$hash[2]] = $hash[3];
			$link_title[$hash[2]] = $hash[8];
			if ($hash[9] == 2)
			{
				$link[$hash[2]] = $hash[10];
			}
			else
			{
				if ( strlen($hash[11]) == 0 )
				{
					$link[$hash[2]] = 'dir.php?id='.$hash[2];
				}
				else
				{
					$link[$hash[2]] = './dir_'.urlencode($hash[11]).'.htm';
				}
			}
		}

		while ($hash[1] > 0)
		{
			$command = "SELECT tindex, tlevel, id, name, in_cat_down, in_cat_center, in_cat_menu, in_cat_left, link_title, istype, link, seo_psu
		    	  	  	FROM {$this->gDBNick}_topic
					 	WHERE id = '$hash[0]'
					 	AND active > 0
					   ";
			$sth = $this->gDB->DoQuery(__LINE__, $command);
			$hash = $this->FetchRow($sth);
			if ($hash[4] == 1 || $hash[5] == 1 || $hash[6] == 1 || $hash[7] == 1 || true )
			{
				$path[$hash[2]] = $hash[3];
				$link_title[$hash[2]] = $hash[8];
				if ($hash[9] == 2)
				{
					$link[$hash[2]] = $hash[10];
				}
				else
				{
					if ( strlen($hash[11]) == 0 )
					{
						$link[$hash[2]] = 'dir.php?id='.$hash[2];
					}
					else
					{
						$link[$hash[2]] = './dir_'.urlencode($hash[11]).'.htm';
					}
				}
			}
		}

		return array($path, $link, $link_title);
	}

	/**
	 * GetCatLeft
	 **/
	function GetCatLeft($mShowToLevel = 2, $mCurrTopicID =-1)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_topic_list";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_topic_list
					SELECT id, name, tlevel, absindex, div_before, istype, link, link_title, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype < 3
					  AND in_cat_left = 1
					  AND (tlevel < '$mShowToLevel' OR id = '$mCurrTopicID' OR tindex = '$mCurrTopicID')
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT tindex, tlevel, id, name
		    	  	  FROM {$this->gDBNick}_topic
					 WHERE id = '$mCurrTopicID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);
		while ($hash[1] > ($mShowToLevel-1))
		{
			$command = "INSERT INTO {$this->gDBNick}_topic_list
						SELECT id, name, tlevel, absindex, div_before, istype, link, link_title, seo_psu, cache_how_goods
		    			FROM {$this->gDBNick}_topic
		   		    	WHERE active = 1
						  AND istype < 3
						  AND in_cat_left = 1
					  	  AND tindex = '$hash[0]'
					   ";
			$this->gDB->DoQuery(__LINE__, $command);

			$command = "SELECT tindex, tlevel, id, name
		    	  	  	FROM {$this->gDBNick}_topic
					 	WHERE id = '$hash[0]'
					   ";
			$sth = $this->gDB->DoQuery(__LINE__, $command);
			$hash = $this->FetchRow($sth);
		}
		$command = "SELECT MAX(tlevel)+1
		    	  	  FROM {$this->gDBNick}_topic_list
					 ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);
		$command="SELECT DISTINCT id, name, tlevel, absindex, div_before, istype, link, link_title, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic_list
				ORDER BY absindex";

		return array($hash[0], $this->gDB->DoQuery(__LINE__, $command));
	}

	/**
	 * GetCatDown
	 **/
	function GetCatDown($mShowToLevel)
	{
		$command = "SELECT id, name, tlevel, absindex, div_before, istype, link, tindex, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype < 3
					  AND in_cat_down = 1
					  AND tlevel < '$mShowToLevel'
			     ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCatExpSearch
	 **/
	function GetExpSearch($mShowToLevel)
	{
		$command = "SELECT id, name, tlevel, absindex, div_before, istype, link, tindex
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype = 0
					  AND ( in_cat_down = 1 OR in_cat_center = 1 OR in_cat_menu = 1 OR in_cat_left = 1 )
					  AND tlevel < '$mShowToLevel'
			     ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCatCenter
	 **/
	function GetCatCenter($mStartTopicID)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_topic_center";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_topic_center
					SELECT id, name, 0 AS tlevel, absindex, div_before, istype, link, tindex, full_descr, link_title, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype < 3
					  AND in_cat_center = 1
					  AND tindex = $mStartTopicID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT id
		    	  	  FROM {$this->gDBNick}_topic_center
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$rows = $this->GetNumRows($sth);

		for ( $i = 1; $i <= $rows; $i++ )
		{
			$hash = $this->FetchRow($sth);
			$command = "INSERT INTO {$this->gDBNick}_topic_center
						SELECT id, name, 1 AS tlevel, absindex, div_before, istype, link, tindex, full_descr, link_title, seo_psu, cache_how_goods
		    			FROM {$this->gDBNick}_topic
		   		    	WHERE active = 1
						  AND istype < 3
						  AND in_cat_center = 1
					  	  AND tindex = '$hash[0]'
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}

		$command = "SELECT id, name, tlevel, absindex, div_before, istype, link, tindex, full_descr, link_title, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic_center
			     ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCatMenu
	 **/
	function GetCatMenu()
	{
		$command = "SELECT id, name, tlevel, absindex, div_before, istype, link, tindex, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype < 3
					  AND in_cat_menu = 1
					  AND tlevel < 2
			     ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCatFull
	 **/
	function GetCatFull()
	{
		$command = "SELECT id, name, tlevel, absindex, div_before, istype, link, tindex, seo_psu
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype < 3
					  AND ( in_cat_down = 1 OR in_cat_center = 1 OR in_cat_menu = 1 OR in_cat_left = 1 )
			     ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetCatForXML
	 **/
	function GetCatForXML()
	{
		$command = "SELECT id, name, tlevel, absindex, div_before, istype, link, tindex, seo_psu
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND in_xml = 1
					  AND istype < 3
			     ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetSubSectionsSimple
	 **/
	function GetSubSectionsSimple($mCurrTopicID =-1)
	{
		$command = "SELECT id, name, tlevel, absindex, full_descr, istype, link, link_title, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype < 3
					  AND tindex = '$mCurrTopicID'
					  AND ( in_cat_down = 1 OR in_cat_center = 1 OR in_cat_menu = 1 OR in_cat_left = 1 )
				 ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetGoodsSection
	 **/
	function GetGoodsSection($mGoodsID)
	{
		$command = "SELECT topic.id, topic.name, topic.tlevel, topic.absindex, topic.full_descr, topic.istype, topic.link, topic.link_title, topic.seo_psu
		    		FROM {$this->gDBNick}_topic topic,
					     {$this->gDBNick}_topic_store topic_store
		   		    WHERE topic_store.topic_id = topic.id
					  AND topic_store.store_id = '$mGoodsID'
					  AND topic.active = 1
					  AND topic.istype < 3
				 ORDER BY topic.absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetSubSectionsSearch
	 **/
	function GetSubSectionsSearch($mApplyFilter)
	{
		// is use filter?
		if ( $mApplyFilter )
		{
		    $table = 'store_tmp_filter';
		}
		else
		{
			$table = 'store_tmp';
		}

		$command = "SELECT DISTINCT topic.id, topic.name, topic.tlevel, topic.absindex, topic.full_descr, topic.istype, topic.link, topic.link_title, topic.seo_psu,
						   topic.cache_how_goods
		    		FROM {$this->gDBNick}_topic topic,
			   			 {$this->gDBNick}_topic_store topic_store,
			         	 {$this->gDBNick}_$table store
		   		    WHERE topic_store.topic_id = topic.id
					  AND topic_store.store_id = store.id
					  AND active = 1
					  AND istype < 3
				 ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetSubSectionsHard
	 **/
	function GetSubSectionsHard($mCurrTopicID =-1)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_sub_topic_list";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_sub_topic_list
				    SELECT id, name, tlevel, absindex, full_descr, istype, link, link_title, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_topic
		   		    WHERE active = 1
					  AND istype < 3
					  AND (tindex = '$mCurrTopicID')
					  AND ( in_cat_down = 1 OR in_cat_center = 1 OR in_cat_menu = 1 OR in_cat_left = 1 )
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT id
		    		FROM {$this->gDBNick}_sub_topic_list
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		while ( $hash = $this->FetchRow($sth) )
		{
			$command = "INSERT INTO {$this->gDBNick}_sub_topic_list
					    SELECT id, name, tlevel, absindex, full_descr, istype, link, link_title, seo_psu, cache_how_goods
			    		FROM {$this->gDBNick}_topic
		   			    WHERE active = 1
						  AND istype < 3
						  AND tindex = '$hash[0]'
						  AND ( in_cat_down = 1 OR in_cat_center = 1 OR in_cat_menu = 1 OR in_cat_left = 1 )
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}
		$command="SELECT DISTINCT id, name, tlevel, absindex, full_descr, istype, link, link_title, seo_psu, cache_how_goods
		    		FROM {$this->gDBNick}_sub_topic_list
				ORDER BY absindex";

		return $this->gDB->DoQuery(__LINE__, $command);
	}


	//***************************************************************************************************
	//
	// 									   		Clients
	//
	//***************************************************************************************************


	/**
	 * AddClient
	 **/
	function AddClient($mLogin, $mPass, $mEmail, $mDate, $mCharset)
	{
		$command = "INSERT INTO {$this->gDBNick}_client
			   		        SET login = '$mLogin',
							    pass = '$mPass',
								email = '$mEmail',
								reg_date = '$mDate',
								last_date = '$mDate',
								charset = '$mCharset',
								rules_allow = '1'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT LAST_INSERT_ID()";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetLastClientID
	 **/
	function GetLastClientID()
	{
		$command = "SELECT MAX(id) FROM {$this->gDBNick}_client";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * AddClientField
	 **/
	function AddClientField($mClientID, $mFieldID, $mValue)
	{
		$command = "INSERT INTO {$this->gDBNick}_client_field_value
			   		        SET client_id = '$mClientID',
							    client_field_id = '$mFieldID',
								value_text = '$mValue'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * RemoveClient
	 **/
	function RemoveClient($mClientID)
	{
		$command = "DELETE FROM {$this->gDBNick}_client
					      WHERE id = '$mClientID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_client_field_value
					      WHERE client_id = '$mClientID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetClient
	 **/
	function GetClient($mClientID)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_client
			   		 WHERE id = '$mClientID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * TestForClientRules
	 **/
	function TestForClientRules($mClientID, $mOrdersSuma)
	{
		$command = "SELECT category_id, set_proc, set_suma
			    	  FROM {$this->gDBNick}_client_rules
			   		 WHERE orders_sum <= '$mOrdersSuma'
				  ORDER BY orders_sum DESC
		             LIMIT 1
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) > 0 )
		{
			$hash = $this->FetchRow($sth);

			$command = "UPDATE {$this->gDBNick}_client
						   SET proc = '$hash[1]',
							   suma = '$hash[2]',
							   category_id = '$hash[0]'
				   		 WHERE id = '$mClientID'
				   	   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}

		return true;
	}

	/**
	 * GetAllClients
	 **/
	function GetAllClients($mInput)
	{
		$where = '';
		$table = '';
		if ( $mInput['id']*1 > 0 )
		{
			$where .= ' AND client.id = '.($mInput['id']*1).' ';
		}
		if ( strlen($mInput['login']) > 0 )
		{
			$where .= ' AND client.login LIKE "%'.addslashes($mInput['login']).'%" ';
		}
		if ( strlen($mInput['email']) > 0 )
		{
			$where .= ' AND client.email LIKE "%'.addslashes($mInput['email']).'%" ';
		}
		if ( strlen($mInput['proc_oper']) > 0 )
		{
			$where .= ' AND client.proc '.$mInput['proc_oper'].' '.(1*str_replace(',','.',$mInput['proc_value'])).' ';
		}
		if ( strlen($mInput['suma_oper']) > 0 )
		{
			$where .= ' AND client.suma '.$mInput['suma_oper'].' '.(1*str_replace(',','.',$mInput['suma_value'])).' ';
		}
		if ( $mInput['category_id'] > 0 )
		{
			$where .= ' AND client.category_id = '.$mInput['category_id'].' ';
		}
		if ( strlen($mInput['orders_suma_oper']) > 0 )
		{
			$where .= ' AND client.orders_suma '.$mInput['orders_suma_oper'].' '.(1*str_replace(',','.',$mInput['orders_suma_value'])).' ';
		}
		if ( strlen($mInput['reg_date_oper']) > 0 )
		{
			$where .= ' AND client.reg_date '.$mInput['reg_date_oper'].' "'.addslashes($mInput['reg_date_value']).'" ';
		}
		if ( strlen($mInput['last_date_oper']) > 0 )
		{
			$where .= ' AND client.last_date '.$mInput['last_date_oper'].' "'.addslashes($mInput['last_date_value']).'" ';
		}
		if ( $mInput['rules_allow'] < 2 && strlen($mInput['rules_allow']) > 0 )
		{
			$where .= ' AND client.rules_allow = '.$mInput['rules_allow'].' ';
		}
		if ( strlen($mInput['field_value']) > 0 )
		{
			$where .= ' AND client_field_value.value_text LIKE "%'.addslashes($mInput['field_value']).'%" ';
			$table = " LEFT JOIN {$this->gDBNick}_client_field_value client_field_value
			                  ON client_field_value.client_id = client.id
							 AND client_field_value.client_field_id = '".$mInput[field_id]."'";
		}

		$command = "SELECT client.*, category.name AS category_name
			    	  FROM (
						   {$this->gDBNick}_client client
						   $table
						   )
				 LEFT JOIN {$this->gDBNick}_category category
						ON category.id = client.category_id
					 WHERE client.id > 1
						   $where
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetClientFields
	 **/
	function GetClientFields()
	{
		$command = "SELECT client_field.*, client_field_set.id
			    	  FROM {$this->gDBNick}_client_field client_field
				 LEFT JOIN {$this->gDBNick}_client_field_set client_field_set
				        ON client_field_set.client_field_id = client_field.id
				  GROUP BY client_field.id
				  ORDER BY client_field.absindex
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}


	/**
	 * GetClientData
	 **/
	function GetClientData($mClientID)
	{
		$command = "SELECT client_field.*, client_field_value.value_text
			    	  FROM {$this->gDBNick}_client_field client_field
				 LEFT JOIN {$this->gDBNick}_client_field_value client_field_value
				        ON client_field_value.client_field_id = client_field.id
					   AND client_field_value.client_id = $mClientID
				  ORDER BY client_field.absindex DESC
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetClientFieldSet
	 **/
	function GetClientFieldSet($mFieldID)
	{
		$command = "SELECT set_value
			    	  FROM {$this->gDBNick}_client_field_set
					 WHERE client_field_id = '$mFieldID'
				  ORDER BY pos
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * ClientLoginExists
	 **/
	function ClientLoginExists($mLogin)
	{
		$command = "SELECT COUNT(*)
			    	  FROM {$this->gDBNick}_client
					 WHERE login = '$mLogin'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetClientPass
	 **/
	function GetClientPass($mEmail)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_client
					 WHERE email = '$mEmail'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchHash($sth);

		return array($this->GetNumRows($sth), $hash[login], $hash[pass], $hash[charset]);
	}

	/**
	 * VerifyEnter
	 **/
	function VerifyEnter($mLogin, $mPass)
	{
		$command = "SELECT id
		    	  	  FROM {$this->gDBNick}_client
					 WHERE BINARY login = '$mLogin'
					   AND BINARY pass = '$mPass'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) == 0 )
		{
		    return 0;
		}
		else
		{
			$hash = $this->FetchHash($sth);

			return $hash['id'];
		}
	}


	/**
	 * AddClientOrderSum
	 **/
	function AddClientOrderSum($mClientID, $mSuma)
	{
		$command = "UPDATE {$this->gDBNick}_client
					   SET orders_suma = orders_suma + $mSuma
			   		 WHERE id = '$mClientID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SaveClientParams
	 **/
	function SaveClientParams($mClientID, $mProc, $mSuma, $mCategiryID, $mOrdersSuma, $mRulesAllow, $mRegDate, $mLastDate)
	{
		$command = "UPDATE {$this->gDBNick}_client
					   SET proc = '$mProc',
						   suma = '$mSuma',
						   category_id = '$mCategiryID',
						   orders_suma = '$mOrdersSuma',
						   rules_allow = '$mRulesAllow',
						   reg_date = '$mRegDate',
						   last_date = '$mLastDate'
			   		 WHERE id = '$mClientID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SaveClientBaseField
	 **/
	function SaveClientBaseField($mClientID, $mField, $mValue)
	{
		$command = "UPDATE {$this->gDBNick}_client
					   SET $mField = '$mValue'
			   		 WHERE id = '$mClientID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SaveClientFieldValue
	 **/
	function SaveClientFieldValue($mClientID, $mFieldID, $mValue)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_client_field_value
					 WHERE client_id = '$mClientID'
		               AND client_field_id = '$mFieldID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) == 0 )
		{
			$command = "INSERT INTO {$this->gDBNick}_client_field_value
						   SET value_text = '$mValue',
			    	           client_id = '$mClientID',
		            	       client_field_id = '$mFieldID'
			   	   	   ";
		}
		else
		{
			$command = "UPDATE {$this->gDBNick}_client_field_value
						   SET value_text = '$mValue'
			    	     WHERE client_id = '$mClientID'
		            	   AND client_field_id = '$mFieldID'
			   	   	   ";
		}
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * RemoveClientFieldValue
	 **/
	function RemoveClientFieldValue($mClientID, $mFieldID)
	{
		$command = "DELETE FROM {$this->gDBNick}_client_field_value
			         WHERE client_id = '$mClientID'
		               AND client_field_id = '$mFieldID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	//***************************************************************************************************
	//
	// 									   		Orders
	//
	//***************************************************************************************************


	/**
	 * GetCurrentBasketGoods
	 **/
	function GetCurrentBasketGoods($mSID, $mStoreID, $mParamValueID, $mModifyCode)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_current_basket
			   		 WHERE sid = '$mSID'
					   AND store_id = '$mStoreID'
					   AND param_value_id = '$mParamValueID'
					   AND modify_code = '$mModifyCode'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetCurrentBasket
	 **/
	function GetCurrentBasket($mSID)
	{
		$command = "SELECT current_basket.*, store.step_order, store.min_order,
						   IF ( current_basket.param_personal > 0, store_add.how, store.how ) AS max_order,
						   store.step_order, store.min_order
			    	  FROM (
						   {$this->gDBNick}_current_basket current_basket,
					  	   {$this->gDBNick}_store store
						   )
				 LEFT JOIN {$this->gDBNick}_store_add store_add
				        ON store_add.store_id = current_basket.store_id
					   AND store_add.param_value_id = current_basket.param_value_id
			   		 WHERE store.id = current_basket.store_id
					   AND sid = '$mSID'
				  ORDER BY current_basket.date_time
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}


	/**
	 * GetMaxOrderGoods
	 **/
	function GetMaxOrderGoods($mStoreID, $mParamValueID)
	{
		$table = 'store';
		if ( $mParamValueID > 0 )
		{
	    	if ( $this->GoodsPersonalValue($mStoreID, $mParamValueID) )
	    	{
	    		$table = 'store_add';
	    	}
		}

		if ( $table == 'store_add' )
		{
			$command = "SELECT how
			              FROM {$this->gDBNick}_store_add
			   			 WHERE store_id = '$mStoreID'
						   AND param_value_id = '$mParamValueID'
				   	   ";
		}
		else
		{
			$command = "SELECT how
			              FROM {$this->gDBNick}_store
			   			 WHERE id = '$mStoreID'
				   	   ";
		}
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}


	/**
	 * PutCurrentBasket
	 **/
	function PutCurrentBasket($mSID, $mStoreID, $mCode, $mTax, $mStatus, $mHow, $mPrice, $mDiscPrice,
							  $mParamValueID, $mParamName, $mParamPersonal, $mParamPrice, $mParamProc,
							  $mParamSuma, $mDateTime, $mModifySuma, $mModifyCode, $mName, $mMeas, $mPartProc,
							  $mProviderID, $mProvPrice)
	{
		$command = "INSERT INTO {$this->gDBNick}_current_basket
			   		   	    SET sid = '$mSID',
							    store_id = '$mStoreID',
								code = '$mCode',
								tax = '$mTax',
								status = '$mStatus',
								how = '$mHow',
								price = '$mPrice',
								disc_price = '$mDiscPrice',
								param_value_id = '$mParamValueID',
								param_name = '$mParamName',
								param_personal = '$mParamPersonal',
								param_price = '$mParamPrice',
								param_proc = '$mParamProc',
								param_suma = '$mParamSuma',
								date_time = '$mDateTime',
								modify_suma = '$mModifySuma',
								modify_code = '$mModifyCode',
								name = '$mName',
								meas = '$mMeas',
							    part_proc = '$mPartProc',
								provider_id = '$mProviderID',
								prov_price = '$mProvPrice'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * UpdateCurrentBasket
	 **/
	function UpdateCurrentBasket($mBasketID, $mHow)
	{
		$command = "UPDATE {$this->gDBNick}_current_basket
			   		   SET how = '$mHow'
				     WHERE id = '$mBasketID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * RemoveCurrentBasket
	 **/
	function RemoveCurrentBasket($mBasketID)
	{
		$command = "DELETE FROM {$this->gDBNick}_current_basket
			   		 	  WHERE id = '$mBasketID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}


	/**
	 * GetDeliveryValue
	 **/
	function GetDeliveryValue($mDeliveryID, $mTax, $mOrderSum)
	{
		$command = "SELECT delivery.id, delivery.name, delivery.descr, delivery.tlevel, delivery.tindex,
						   delivery.absindex, delivery.proc, delivery.suma, delivery.free_from, delivery.isfolder,
						   delivery_multiplex.id, delivery_multiplex.multiplex,
						   delivery_cost.id, delivery_cost.cost
		    		  FROM {$this->gDBNick}_delivery delivery
				 LEFT JOIN {$this->gDBNick}_delivery_multiplex delivery_multiplex
				        ON delivery.id = delivery_multiplex.delivery_id
					   AND delivery_multiplex.tax_from <= '$mTax'
				 LEFT JOIN {$this->gDBNick}_delivery_cost delivery_cost
				        ON delivery.id = delivery_cost.delivery_id
					   AND delivery_cost.sum_from <= '$mOrderSum'
				 	 WHERE delivery.id = '$mDeliveryID'
				  ORDER BY delivery_multiplex.tax_from DESC, delivery_cost.sum_from DESC
				     LIMIT 1
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetDelivery
	 **/
	function GetDelivery()
	{
		$command = "SELECT delivery.id, delivery.name, delivery.descr, delivery.tlevel, delivery.tindex,
						   delivery.absindex, delivery.proc, delivery.suma, delivery.free_from, delivery.isfolder,
						   delivery_multiplex.id, delivery_cost.id
		    		  FROM {$this->gDBNick}_delivery delivery
				 LEFT JOIN {$this->gDBNick}_delivery_multiplex delivery_multiplex
				        ON delivery.id = delivery_multiplex.delivery_id
				 LEFT JOIN {$this->gDBNick}_delivery_cost delivery_cost
				        ON delivery.id = delivery_cost.delivery_id
	           	  GROUP BY delivery.id
				  ORDER BY delivery.absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetDeliveryMultiplex
	 **/
	function GetDeliveryMultiplex($mDeliveryID, $mTax)
	{
		$command = "SELECT multiplex
		    		  FROM {$this->gDBNick}_delivery_multiplex
					 WHERE delivery_id = '$mDeliveryID'
					   AND tax_from <= '$mTax'
				  ORDER BY tax_from DESC
				     LIMIT 1
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetDeliveryCost
	 **/
	function GetDeliveryCost($mDeliveryID, $mOrderSum)
	{
		$command = "SELECT cost
		    		  FROM {$this->gDBNick}_delivery_cost
					 WHERE delivery_id = '$mDeliveryID'
					   AND sum_from <= '$mOrderSum'
				  ORDER BY sum_from DESC
				     LIMIT 1
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetCondition
	 **/
	function GetCondition()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_condition
				  ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPayment
	 **/
	function GetPayment()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_payment
				  ORDER BY absindex DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPaymentValue
	 **/
	function GetPaymentValue($mPaymentID)
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_payment
				     WHERE id = '$mPaymentID'
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * MakeOrderCode
	 **/
	function MakeOrderCode()
	{
		$command = "UPDATE {$this->gDBNick}_shop_values
				       SET key_value=LAST_INSERT_ID(key_value+1)
				     WHERE key_name='order_num'
					";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT LAST_INSERT_ID()";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * TestPaymentDelivery
	 **/
	function TestPaymentDelivery($mPaymentID, $mDeliveryID)
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_payment_delivery
				     WHERE payment_id = '$mPaymentID'
					   AND delivery_id = '$mDeliveryID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return ( $this->GetNumRows($sth) == 0 );
	}

	//***************************************************************************************************
	//
	// 									   		Archive orders and clients
	//
	//***************************************************************************************************

	/**
	 * AddArchiveOrder
	 **/
	function AddArchiveOrder($mSSID, $mCode, $mClientID, $mPartnerID, $mDateTime, $mManagerID = 0)
	{
		$command = "INSERT INTO {$this->gDBNick}_archive_order
			   		        SET sid = '$mSSID',
							    code = '$mCode',
							    client_id = '$mClientID',
							    manager_id = '$mManagerID',
								partner_id = '$mPartnerID',
								date_time = '$mDateTime',
								status = '0',
								get_payment = '1',
							    update_time = '$mDateTime'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT LAST_INSERT_ID()";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * UpdateArchiveOrder
	 **/
	function UpdateArchiveOrder($mOrderID, $mPartnerID, $mManagerID, $mDateTime, $mUpdateTime)
	{
		$command = "UPDATE {$this->gDBNick}_archive_order
			   		   SET manager_id = '$mManagerID',
						   partner_id = '$mPartnerID',
						   date_time = '$mDateTime',
						   update_time = '$mUpdateTime'
					 WHERE id = '$mOrderID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_archive_basket WHERE order_id = '$mOrderID'";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_archive_client_field WHERE order_id = '$mOrderID'";
		$this->gDB->DoQuery(__LINE__, $command);

		return $mUpdateTime;
	}


	/**
	 * SaveArchiveClientField
	 **/
	function SaveArchiveClientField($mOrderID, $mFieldID, $mName, $mLevel, $mAbsIndex, $mIsFolder, $mValue)
	{
		$command = "INSERT INTO {$this->gDBNick}_archive_client_field
			   		   	    SET order_id = '$mOrderID',
							    field_id = '$mFieldID',
								name = '$mName',
								tlevel = '$mLevel',
								absindex = '$mAbsIndex',
								isfolder = '$mIsFolder',
								value_text = '$mValue'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * AddArchiveBasket
	 **/
	function AddArchiveBasket($mOrderID, $mStoreID, $mCode, $mMeas, $mName, $mParamName, $mParamPersonal, $mParamValueID, $mPrice, $mNewPrice,
							  $mStatus, $mHowOrdered, $mPartProc, $mPartValue, $mModifyCode, $mProviderID, $mProvPrice, $mTax, $mInsideDisc)
	{
		$command = "INSERT INTO {$this->gDBNick}_archive_basket
			   		   	    SET order_id = '$mOrderID',
								store_id = '$mStoreID',
							    code = '$mCode',
								meas = '$mMeas',
								name = '$mName',
								param_name = '$mParamName',
								param_personal = '$mParamPersonal',
								param_value_id = '$mParamValueID',
								price = '$mPrice',
								new_price = '$mNewPrice',
								status = '$mStatus',
								how = '$mHowOrdered',
								part_proc = '$mPartProc',
								part_value = '$mPartValue',
							    modify_code = '$mModifyCode',
								provider_id = '$mProviderID',
							    prov_price = '$mProvPrice',
								tax = '$mTax',
								inside_disc = '$mInsideDisc'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SaveArchiveOrder
	 **/
	function SaveArchiveOrder($mOrderID, $mDiscSum, $mCondName, $mCondCost, $mDelivName, $mDelivCost,
							  $mPaymentName, $mPaymentCost, $mPaymentCurr, $mPaymentID, $mItogoSum,
							  $mTotalSum, $mNotice, $mPartnerSum, $mDelivID, $mCondCode )
	{
		$command = "UPDATE {$this->gDBNick}_archive_order
			   		   SET disc_sum = '$mDiscSum',
						   cond_name = '$mCondName',
						   cond_cost = '$mCondCost',
						   deliv_name = '$mDelivName',
						   deliv_cost = '$mDelivCost',
						   payment_name = '$mPaymentName',
						   payment_cost = '$mPaymentCost',
						   payment_curr = '$mPaymentCurr',
						   payment_id = '$mPaymentID',
						   itogo_sum = '$mItogoSum',
						   total_sum = '$mTotalSum',
						   notice =	 '$mNotice',
						   partner_sum = '$mPartnerSum',
						   deliv_id = '$mDelivID',
						   cond_code = '$mCondCode'
					WHERE id = '$mOrderID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * UpdateArchiveNotice
	 **/
	function UpdateArchiveNotice($mOrderID, $mNotice, $mUpdateTime)
	{
		$command = "UPDATE {$this->gDBNick}_archive_order
			   		   SET inside_note = '$mNotice',
						   update_time = '$mUpdateTime'
					WHERE id = '$mOrderID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * UpdateArchiveStatus
	 **/
	function UpdateArchiveStatus($mOrderID, $mStatus, $mUpdateTime)
	{
		$command = "UPDATE {$this->gDBNick}_archive_order
			   		   SET status = '$mStatus',
						   update_time = '$mUpdateTime'
					WHERE id = '$mOrderID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * UpdateArchivePayment
	 **/
	function UpdateArchivePayment($mOrderID, $mPayment, $mUpdateTime)
	{
		$command = "UPDATE {$this->gDBNick}_archive_order
			   		   SET get_payment = '$mPayment',
						   update_time = '$mUpdateTime'
					WHERE id = '$mOrderID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * SetOrderManager
	 **/
	function SetOrderManager($mOrderID, $mManagerID, $mUpdateTime)
	{
		$command = "UPDATE {$this->gDBNick}_archive_order
			   		   SET manager_id = '$mManagerID',
						   update_time = '$mUpdateTime'
					WHERE id = '$mOrderID'
					  AND manager_id = 0
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * RemoveArchiveOrder
	 **/
	function RemoveArchiveOrder($mOrderID)
	{
		$command = "DELETE FROM {$this->gDBNick}_archive_order WHERE id = '$mOrderID'";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_archive_basket WHERE order_id = '$mOrderID'";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_archive_client_field WHERE order_id = '$mOrderID'";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}


	/**
	 * GetArchiveOrders
	 **/
	function GetArchiveOrders($mWhat = '', $mSearch = '')
	{
		switch($mWhat)
		{
			case 'order_code' :
				$table = '';
				$where = " AND archive_order.code LIKE '%$mSearch%' ";
				break;
			case 'order_datetime' :
				$table = '';
				list($start_datetime, $end_datetime) = explode(';', $mSearch);
				$where = " AND archive_order.date_time >= '$start_datetime' AND  archive_order.date_time <= '$end_datetime' ";
				break;
			case 'goods_code' :
				$table = " LEFT JOIN {$this->gDBNick}_archive_basket archive_basket
						   ON archive_order.id = archive_basket.order_id ";
				$where = " AND archive_basket.code LIKE '%$mSearch%'
						   GROUP BY archive_order.id ";
				break;
			case 'goods_name' :
				$table = " LEFT JOIN {$this->gDBNick}_archive_basket archive_basket
						   ON archive_order.id = archive_basket.order_id
						 ";
				$where = " AND archive_basket.name LIKE '%$mSearch%'
						   GROUP BY archive_order.id ";
				break;
			case 'client_login' :
				$table = " LEFT JOIN {$this->gDBNick}_archive_client_field archive_client_field
						   ON archive_order.id = archive_client_field.order_id AND archive_client_field.field_id = -3";
				$where = " AND archive_client_field.value_text LIKE '%$mSearch%'
						   GROUP BY archive_order.id ";
				break;
			case 'status0' :
				$table = '';
				$where = " AND archive_order.status = 0 ";
				break;
			case 'status1' :
				$table = '';
				$where = " AND archive_order.status = 1 ";
				break;
			case 'status2' :
				$table = '';
				$where = " AND archive_order.status = 2 ";
				break;
			case 'status3' :
				$table = '';
				$where = " AND archive_order.status = 3 ";
				break;
			case 'status4' :
				$table = '';
				$where = " AND archive_order.status = 4 ";
				break;
			case 'payment1' :
				$table = '';
				$where = " AND archive_order.get_payment = 1 ";
				break;
			case 'payment2' :
				$table = '';
				$where = " AND archive_order.get_payment = 2 ";
				break;
			default:
				$table = '';
				$where = '';
		} // switch

		$command="SELECT archive_order.*, partner.login AS partner_login, manager.name AS manager_name
		    	    FROM (
						 {$this->gDBNick}_archive_order archive_order
						 $table
						 )
			   LEFT JOIN {$this->gDBNick}_partner partner
					  ON archive_order.partner_id = partner.id
			   LEFT JOIN {$this->gDBNick}_manager manager
					  ON archive_order.manager_id = manager.id
	           	   WHERE archive_order.id > '$mLastOrderID'
						 $where
				ORDER BY archive_order.date_time DESC
				 ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetArchiveOrder
	 **/
	function GetArchiveOrder($mOrderID, $mOrderSID = '')
	{
		if ( $mOrderSID == '' )
		{
			$where = " archive_order.id = '$mOrderID' ";
		}
		else
		{
			$where = " archive_order.sid = '$mOrderSID' ";
		}
		$command="SELECT archive_order.*, partner.login AS partner_login, manager.name AS manager_name
		    	    FROM {$this->gDBNick}_archive_order archive_order
			   LEFT JOIN {$this->gDBNick}_partner partner
					  ON archive_order.partner_id = partner.id
			   LEFT JOIN {$this->gDBNick}_manager manager
					  ON archive_order.manager_id = manager.id
	           	   WHERE $where
				 ";

		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetArchiveClientField
	 **/
	function GetArchiveClientField($mOrderID)
	{
		$command="SELECT *
		    	    FROM {$this->gDBNick}_archive_client_field
	           	   WHERE order_id = '$mOrderID'
				ORDER BY absindex
				 ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetArchiveGoods
	 **/
	function GetArchiveGoods($mOrderID)
	{
		$command="SELECT *
		    	    FROM {$this->gDBNick}_archive_basket
	           	   WHERE order_id = '$mOrderID'
				ORDER BY id
				 ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetAllOrdersClient
	 **/
	function GetAllOrdersClient($mClientID)
	{
		$command="SELECT archive_order.sid, archive_order.code, archive_order.total_sum, archive_order.date_time,
						 archive_order.payment_id, archive_order.id, archive_order.status, archive_order.get_payment,
						 payment.online
		    	    FROM {$this->gDBNick}_archive_order archive_order
		       LEFT JOIN {$this->gDBNick}_payment payment
		              ON archive_order.payment_id = payment.id
	           	   WHERE archive_order.client_id = '$mClientID'
				ORDER BY archive_order.date_time DESC
				 ";

		return $this->gDB->DoQuery(__LINE__, $command);

	}

	/**
	 * GetAllOrdersPartner
	 **/
	function GetAllOrdersPartner($mPartnerID)
	{
		$command="SELECT archive_order.id, archive_order.code, archive_order.date_time, archive_order.status, archive_order.partner_sum
		    	    FROM {$this->gDBNick}_archive_order archive_order,
						 {$this->gDBNick}_archive_basket archive_basket
	           	   WHERE archive_order.id = archive_basket.order_id
					 AND archive_basket.status IN ( 0, 3 )
					 AND archive_order.partner_id = '$mPartnerID'
				GROUP BY archive_order.id
				ORDER BY date_time DESC
				 ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	//***************************************************************************************************
	//
	// 									   		Managers
	//
	//***************************************************************************************************

	/**
	 * ManagerLogin
	 **/
	function ManagerLogin($mLogin, $mPass)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_manager
					 WHERE BINARY login = '$mLogin'
					   AND BINARY pass = '$mPass'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) == 0 )
		{
		    return false;
		}
		else
		{
			return $this->FetchHash($sth);
		}
	}

	/**
	 * GetManager
	 **/
	function GetManager($mManagerID)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_manager
			   		 WHERE id = '$mManagerID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	//***************************************************************************************************
	//
	// 						   		Additional: voting system
	//
	//***************************************************************************************************

	/**
	 * GetFirstPageVote
	 **/
	function GetFirstPageVote($mDate)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_voting
					 WHERE isfolder = 0
					   AND start_date <= '$mDate'
				       AND end_date >= '$mDate'
				  ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetVoteList
	 **/
	function GetVoteList($mVoteID)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_vote_list
					 WHERE vote_id = $mVoteID
				  ORDER BY pos
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetVoteDescr
	 **/
	function GetVoteDescr($mVoteListID)
	{
		$command = "SELECT value_descr
		    	  	  FROM {$this->gDBNick}_vote_list
					 WHERE id = $mVoteListID
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) > 0 )
		{
			$hash = $this->FetchRow($sth);
			return $hash[0];
		}

		return '';
	}

	/**
	 * AddVote
	 **/
	function AddVote($mVoteList, $mSID, $mDateTime)
	{
		$vote = explode(',', $mVoteList);
		$command = "SELECT vote_id
		    	  	  FROM {$this->gDBNick}_vote_list
					 WHERE id = $vote[0]
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		$command = "SELECT DISTINCT vote_list.vote_id
		    	  	  FROM {$this->gDBNick}_vote_list vote_list,
						   {$this->gDBNick}_vote_pool vote_pool
					 WHERE vote_list.id = vote_pool.vote_list_id
					   AND sid = '$mSID'
					   AND vote_list.vote_id = $hash[0]
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		if ( $this->GetNumRows($sth) > 0 ) return false;

		foreach ( $vote as $id )
		{
			$command = "INSERT
						  INTO {$this->gDBNick}_vote_pool
						   SET vote_list_id = $id,
							   sid = '$mSID',
							   date_time = '$mDateTime'
				   	   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}

		return true;
	}

	/**
	 * GetVote
	 **/
	function GetVote($mVoteID)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_voting
					 WHERE id = $mVoteID
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchRow($sth);
	}

	/**
	 * GetVoteResultSum
	 **/
	function GetVoteResultSum($mVoteID)
	{
		$command = "SELECT vote_pool.vote_list_id
		    	  	  FROM {$this->gDBNick}_vote_list vote_list,
						   {$this->gDBNick}_vote_pool vote_pool
					 WHERE vote_list.id = vote_pool.vote_list_id
					   AND vote_list.vote_id = $mVoteID
				  GROUP BY vote_pool.sid
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->GetNumRows($sth);
	}

	/**
	 * GetVoteResult
	 **/
	function GetVoteResult($mVoteID)
	{
		$command = "SELECT vote_list.*, COUNT(vote_pool.vote_list_id)
		    	  	  FROM {$this->gDBNick}_vote_list vote_list
				 LEFT JOIN {$this->gDBNick}_vote_pool vote_pool
						ON vote_list.id = vote_pool.vote_list_id
					 WHERE vote_list.vote_id = $mVoteID
				  GROUP BY vote_list.id
				  ORDER BY vote_list.pos
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetVoteTree
	 **/
	function GetVoteTree()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_voting
				  ORDER BY absindex
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	//***************************************************************************************************
	//
	// 						   		Additional: news system & subscribe
	//
	//***************************************************************************************************

	/**
	 * GetFirstPageNews
	 **/
	function GetFirstPageNews()
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_news
					 WHERE isfolder = 0
					   AND ishidden = 0
				       AND istype = 0
				  ORDER BY absindex
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNewsTree
	 **/
	function GetNewsTree()
	{
		$command = "SELECT *
		    		  FROM {$this->gDBNick}_news
				  ORDER BY absindex
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNewsList
	 **/
	function GetNewsList($mNewsID, $mHow)
	{
		$command = "SELECT id, name, date_time, descr, content, attach
		    	  	  FROM {$this->gDBNick}_news_content
					 WHERE news_id = $mNewsID
				  ORDER BY date_time DESC
				     LIMIT $mHow
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNews
	 **/
	function GetNews($mID)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_news
					 WHERE id = $mID
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetNewsContent
	 **/
	function GetNewsContent($mID)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_news_content
					 WHERE id = $mID
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * NewSubscribe
	 **/
	function NewSubscribe($mNewsID, $mEmail, $mCharset, $mDate)
	{
		$command = "DELETE
					  FROM {$this->gDBNick}_subscriber
					 WHERE news_id = $mNewsID AND email = '$mEmail'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT
					  INTO {$this->gDBNick}_subscriber
					   SET news_id = $mNewsID,
						   email = '$mEmail',
						   charset = '$mCharset',
						   reg_date = '$mDate'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}


	/**
	 * UnSubscribe
	 **/
	function UnSubscribe($mEmail, $mNewsList)
	{
		$command = "SELECT *
					  FROM {$this->gDBNick}_subscriber
					 WHERE email = '$mEmail'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) == 0 )	return false;

		$command = "DELETE
					  FROM {$this->gDBNick}_subscriber
					 WHERE news_id IN ( $mNewsList ) AND email = '$mEmail'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetNewsRSS
	 **/
	function GetNewsRSS()
	{
		$command = "SELECT news.name,
						   news_content.id, news_content.name, news_content.descr, news_content.content,
						   news_content.attach, news_content.date_time
		    	  	  FROM {$this->gDBNick}_news news,
						   {$this->gDBNick}_news_content news_content
					 WHERE news.id = news_content.news_id
					   AND news.rss = 1
					   AND news.istype = 0
				  ORDER BY news_content.date_time DESC
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNewsForMail
	 **/
	function GetNewsForMail($mFromDateTime)
	{

		$command = "SELECT news.id, news.name, news.for_buyer, news.ishtml, news.istype, news.isperiod,
						   news_content.id AS cont_id, news_content.date_time, news_content.name AS cont_name,
						   news_content.attach, news_content.descr, news_content.content
		    		  FROM {$this->gDBNick}_news news
				 LEFT JOIN {$this->gDBNick}_news_content news_content
						ON news.id = news_content.news_id
					   AND news_content.date_time > '$mFromDateTime'
				  ORDER BY news.istype, news_content.date_time DESC
					";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetNewsEmailList
	 **/
	function GetNewsEmailList($mNewsID, $mWithBuyer)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_email_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_email_tmp
					SELECT email, charset, reg_date
					  FROM {$this->gDBNick}_subscriber
					 WHERE news_id = $mNewsID
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		if ( $mWithBuyer )
		{
			$command = "INSERT INTO {$this->gDBNick}_email_tmp
						SELECT email, charset, last_date
		    			  FROM {$this->gDBNick}_client
		   		    	 WHERE email <> ''
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}

		$command = "SELECT DISTINCT email, charset
		    		  FROM {$this->gDBNick}_email_tmp
				  ORDER BY charset, reg_date DESC
					";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		$command = "DROP TABLE {$this->gDBNick}_email_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		return $sth;
	}

	//***************************************************************************************************
	//
	// 									   		Additional: Partner programm
	//
	//***************************************************************************************************

	/**
	 * GetPartnerMain
	 **/
	function GetPartnerMain()
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_partner_main
					 WHERE id = 1
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * GetPartnerCookieDays
	 **/
	function GetPartnerCookieDays()
	{
		$command = "SELECT cookie_days
		    	  	  FROM {$this->gDBNick}_partner_main
					 WHERE id = 1
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetPartnerClickList
	 **/
	function GetPartnerClickList()
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_partner_click
				  ORDER BY click_to_buy
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerPaymentList
	 **/
	function GetPartnerPaymentList()
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_partner_payment
				  ORDER BY pos
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerMailList
	 **/
	function GetPartnerMailList()
	{
		$command = "SELECT email, charset
		    	  	  FROM {$this->gDBNick}_partner
				  ORDER BY charset
				   ";

		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * AddPartner
	 **/
	function AddPartner($mLogin, $mPass, $mEmail, $mDate, $mCharset, $mWorkRegime, $mName, $mTown, $mPhones, $mSite, $mSiteDescr)
	{
		$command = "INSERT INTO {$this->gDBNick}_partner
			   		        SET login = '$mLogin',
							    pass = '$mPass',
								email = '$mEmail',
								reg_date = '$mDate',
								last_date = '$mDate',
								charset = '$mCharset',
								name = '$mName',
							    town = '$mTown',
								phones = '$mPhones',
								site = '$mSite',
								site_descr = '$mSiteDescr',
							    mult_proc = 1,
								mult_click = 1,
								active = 1,
								account = 0,
								status = 0,
								work_regime = $mWorkRegime
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT LAST_INSERT_ID()";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * PartnerLoginExists
	 **/
	function PartnerLoginExists($mLogin)
	{
		$command = "SELECT COUNT(*)
			    	  FROM {$this->gDBNick}_partner
					 WHERE login = '$mLogin'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		return $hash[0];
	}

	/**
	 * GetPartnerStatus
	 **/
	function GetPartnerStatus($mPartnerID)
	{
		$command = "SELECT status
			    	  FROM {$this->gDBNick}_partner
					 WHERE id = '$mPartnerID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) > 0 )
		{
			$hash = $this->FetchRow($sth);
			return $hash[0];
		}
		else
		{
			return -1;
		}
	}

	/**
	 * RemovePartner
	 **/
	function RemovePartner($mPartnerID)
	{
		$command = "DELETE FROM {$this->gDBNick}_partner
					      WHERE id = '$mPartnerID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetPartner
	 **/
	function GetPartner($mPartnerID)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_partner
			   		 WHERE id = '$mPartnerID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		return $this->FetchHash($sth);
	}

	/**
	 * PartnerVerifyEnter
	 **/
	function PartnerVerifyEnter($mLogin, $mPass)
	{
		$command = "SELECT id
		    	  	  FROM {$this->gDBNick}_partner
					 WHERE BINARY login = '$mLogin'
					   AND BINARY pass = '$mPass'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->GetNumRows($sth) == 0 )
		{
		    return 0;
		}
		else
		{
			$hash = $this->FetchHash($sth);

			return $hash['id'];
		}
	}

	/**
	 * GetPartnerPass
	 **/
	function GetPartnerPass($mEmail)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_partner
					 WHERE email = '$mEmail'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchHash($sth);

		return array($this->GetNumRows($sth), $hash['login'], $hash['pass'], $hash['charset']);
	}

	/**
	 * UpdatePartnerField
	 **/
	function UpdatePartnerField($mPartnerID, $mField, $mValue)
	{
		$command = "UPDATE {$this->gDBNick}_partner
					   SET $mField = '$mValue'
			   		 WHERE id = '$mPartnerID'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * RemovePartner
	 **/
	function RemoveParner($mPartnerID)
	{
		$command = "DELETE FROM {$this->gDBNick}_partner
					      WHERE id = '$mPartnerID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_stat
					      WHERE partner_id = '$mPartnerID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_stat_days
					      WHERE partner_id = '$mPartnerID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_stat_month
					      WHERE partner_id = '$mPartnerID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "DELETE FROM {$this->gDBNick}_partner_account
					      WHERE partner_id = '$mPartnerID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * AddPartnerAccount
	 **/
	function AddPartnerAccount($mPartnerID, $mDateTime, $mOperate, $mSuma, $mNotice, $mAccountSave = true)
	{
		$command = "LOCK TABLES {$this->gDBNick}_partner WRITE";
	   	$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT account
			    	  FROM {$this->gDBNick}_partner
					 WHERE id = '$mPartnerID'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->FetchRow($sth);

		if ( $mAccountSave )
		{
			$command = "UPDATE {$this->gDBNick}_partner
						   SET account = account + $mSuma
			   			 WHERE id = '$mPartnerID'
			   	   	   ";
			$this->gDB->DoQuery(__LINE__, $command);

	   		$account = $mSuma + $hash[0];
		}
		else
		{
			$account = $hash[0];
		}

		$command = "UNLOCK TABLES";
   		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_partner_account
					   		SET partner_id = '$mPartnerID',
							    date_time = '$mDateTime',
								operate = '$mOperate',
								suma = '$mSuma',
								account = $account,
								notice = '$mNotice'
			   	   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetPartnerAccount
	 **/
	function GetPartnerAccount($mPartnerID)
	{
		$command = "SELECT *
			    	  FROM {$this->gDBNick}_partner_account
					 WHERE partner_id = '$mPartnerID'
				  ORDER BY date_time DESC
				   ";


		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetAllPartners
	 **/
	function GetAllPartners($mInput)
	{
		$where = '';
		if ( strlen($mInput['id']) > 0 ) $where .= ' AND partner.id = '.($mInput['id']*1).' ';
		if ( strlen($mInput['login']) > 0 ) $where .= ' AND partner.login LIKE "%'.addslashes($mInput['login']).'%" ';
		if ( strlen($mInput['name']) > 0 ) $where .= ' AND partner.name LIKE "%'.addslashes($mInput['name']).'%" ';
		if ( strlen($mInput['email']) > 0 ) $where .= ' AND partner.email LIKE "%'.addslashes($mInput['email']).'%" ';
		if ( strlen($mInput['site']) > 0 ) $where .= ' AND partner.site LIKE "%'.addslashes($mInput['site']).'%" ';
		if ( strlen($mInput['town']) > 0 ) $where .= ' AND partner.town LIKE "%'.addslashes($mInput['town']).'%" ';
		if ( strlen($mInput['active']) > 0  && $mInput['active'] != -1 ) $where .= ' AND partner.active = '.($mInput['active']*1).' ';
		if ( strlen($mInput['work_regime']) > 0  && $mInput['work_regime'] != -1 ) $where .= ' AND partner.work_regime = '.($mInput['work_regime']*1).' ';
		if ( strlen($mInput['status']) > 0  && $mInput['status'] != -1 ) $where .= ' AND partner.status = '.($mInput['status']*1).' ';
		if ( strlen($mInput['mult_proc_oper']) > 0 )
		{
			$where .= ' AND partner.mult_proc '.$mInput['mult_proc_oper'].' '.(1*str_replace(',','.',$mInput['mult_proc_value'])).' ';
		}
		if ( strlen($mInput['mult_click_oper']) > 0 )
		{
			$where .= ' AND partner.mult_click '.$mInput['mult_click_oper'].' '.(1*str_replace(',','.',$mInput['mult_click_value'])).' ';
		}
		if ( strlen($mInput['account_oper']) > 0 )
		{
			$where .= ' AND partner.account '.$mInput['account_oper'].' '.(1*str_replace(',','.',$mInput['account_value'])).' ';
		}
		if ( strlen($mInput['reg_date_oper']) > 0 )
		{
			$where .= ' AND partner.reg_date '.$mInput['reg_date_oper'].' "'.addslashes($mInput['reg_date_value']).'" ';
		}
		if ( strlen($mInput['last_date_oper']) > 0 )
		{
			$where .= ' AND partner.last_date '.$mInput['last_date_oper'].' "'.addslashes($mInput['last_date_value']).'" ';
		}

		$command = "SELECT partner.*
			    	  FROM {$this->gDBNick}_partner partner
					 WHERE partner.id > 0
						   $where
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerNeedPayClick
	 **/
	function GetPartnerNeedPayClick($mLimit, $mMonth, $mTestWorkRegime)
	{
		$work_regime = ( $mTestWorkRegime ) ? 'AND partner.work_regime = 1' : '';
		$command = "SELECT partner.mult_click, partner_stat_month.*
					  FROM {$this->gDBNick}_partner partner,
						   {$this->gDBNick}_partner_stat_month partner_stat_month
					 WHERE partner.id = partner_stat_month.partner_id
					   AND partner_stat_month.pay_was_calc = 0
					   AND partner_stat_month.ismonth = '$mMonth'
					       $work_regime
				  ORDER BY reg_date
					 LIMIT $mLimit
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);

	}

	/**
	 * GetPartnerOrdersCount
	 **/
	function GetPartnerOrdersCount($mPartnerID, $mIsMonth, $mClickHow)
	{
		$command = "SELECT COUNT(id)
					  FROM {$this->gDBNick}_archive_order
					 WHERE partner_id = $mPartnerID
					   AND ( status = 3 OR get_payment = 2 )
					   AND date_time >= '$mIsMonth'
					   AND date_time < DATE_ADD('$mIsMonth', INTERVAL 1 MONTH)
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$order = $this->FetchRow($sth);

		if ( $order[0] == 0 ) return array(0, 0);

		$command = "SELECT suma*$mClickHow/1000
					  FROM {$this->gDBNick}_partner_click
					 WHERE click_to_buy >= ($mClickHow/$order[0])
				  ORDER BY click_to_buy ASC
					 LIMIT 1
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$suma = $this->FetchRow($sth);

		return array($suma[0], $order[0]);
	}

	/**
	 * FixPartnerPay
	 **/
	function FixPartnerPay($mPartnerID, $mMonth)
	{
		$command = "UPDATE {$this->gDBNick}_partner_stat_month
					   SET pay_was_calc = 1
					 WHERE partner_id = $mPartnerID
					   AND ismonth = '$mMonth'
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);

	}

	/**
	 * GetPartnerBlockSite
	 **/
	function GetPartnerBlockSite()
	{
		$command = "SELECT domen
					  FROM {$this->gDBNick}_partner_block_site
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);

	}

	/**
	 * SavePartnerStat
	 **/
	function SavePartnerStat($mPartnerID, $mDateTime, $mDate, $mMonth, $mIP, $mFromPage, $mCookieID, $mClickAccept)
	{
		// Full stat
		$command = "INSERT INTO {$this->gDBNick}_partner_stat
			   		        SET partner_id = '$mPartnerID',
							    date_time = '$mDateTime',
								ip = '$mIP',
								from_page = '$mFromPage',
								cookie_id = '$mCookieID'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		// Days stat
		$command = "UPDATE {$this->gDBNick}_partner_stat_days
					   SET click_all = click_all + 1,
						   click_accept = click_accept + $mClickAccept
					 WHERE partner_id = $mPartnerID
		 			   AND isday = '$mDate'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_partner_stat_days
				   		        SET partner_id = '$mPartnerID',
								    isday = '$mDate',
									click_all = 1,
									click_accept = '$mClickAccept'
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}

		// Month stat
		$command = "UPDATE {$this->gDBNick}_partner_stat_month
					   SET click_all = click_all + 1,
						   click_accept = click_accept + $mClickAccept
					 WHERE partner_id = $mPartnerID
		 			   AND ismonth = '$mMonth'
				   ";
		$this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetAffectedRows() == 0 )
		{
			$this->gDB->SetSilentError();
			$command = "INSERT INTO {$this->gDBNick}_partner_stat_month
				   		        SET partner_id = '$mPartnerID',
								    ismonth = '$mMonth',
									click_all = 1,
									click_accept = '$mClickAccept'
					   ";
			$this->gDB->DoQuery(__LINE__, $command);
			$this->gDB->SilentErrorResult();
		}
		return true;
	}

	/**
	 * SavePartnerStatCommon
	 **/
	function SavePartnerStatCommon($mDay)
	{
		$command = "SELECT isday
		    	  	  FROM {$this->gDBNick}_partner_stat_common
					 WHERE isday = '$mDay'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		if ( $this->gDB->GetNumRows($sth) > 0 ) return false;

		$command = "SELECT COUNT(*) AS part_total,
						   SUM( IF(reg_date = '$mDay', 1, 0) ) AS part_reg,
						   SUM( IF(status = 0, 1, 0) ) AS part_status0,
						   SUM( IF(status = 1, 1, 0) ) AS part_status1,
						   SUM( IF(status = 2, 1, 0) ) AS part_status2,
						   SUM( IF(status = 3, 1, 0) ) AS part_status3,
						   SUM( IF(active = 1, 1, 0) ) AS part_active,
						   SUM(account) AS part_account_total
		    	  	  FROM {$this->gDBNick}_partner
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = $this->gDB->FetchHash($sth);

		$command = "SELECT COUNT(*) AS orders_all,
						   SUM(total_sum) AS orders_all_sum,
						   SUM( IF(partner_id > 0, 1, 0) ) AS orders_part,
						   SUM( IF(partner_id > 0, total_sum, 0) ) AS orders_part_sum
		    	  	  FROM {$this->gDBNick}_archive_order
					 WHERE date_time BETWEEN '$mDay 00:00:00' AND '$mDay 23:59:59'
					   AND status < 4
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = array_merge($hash, $this->gDB->FetchHash($sth));

		$command = "SELECT SUM(click_all) AS click_all,
						   SUM(click_accept) AS click_accept
		    	  	  FROM {$this->gDBNick}_partner_stat_days
					 WHERE isday = '$mDay'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = array_merge($hash, $this->gDB->FetchHash($sth));

		$command = "SELECT SUM( IF(operate = 0, 1, 0) ) AS account_oper0,
						   SUM( IF(operate = 1, 1, 0) ) AS account_oper1,
						   SUM( IF(operate = 2, 1, 0) ) AS account_oper2,
						   SUM( IF(operate = 0, suma, 0) ) AS account_oper0_sum,
						   ABS( SUM( IF(operate = 1, suma, 0) ) ) AS account_oper1_sum,
						   ABS( SUM( IF(operate = 2, suma, 0) ) ) AS account_oper2_sum
		    	  	  FROM {$this->gDBNick}_partner_account
					 WHERE date_time BETWEEN '$mDay 00:00:00' AND '$mDay 23:59:59'
				   ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);
		$hash = array_merge($hash, $this->gDB->FetchHash($sth));

		foreach ($hash as $key => $value) $hash[$key] = $value*1;

		$command = "INSERT INTO {$this->gDBNick}_partner_stat_common
			   		        SET isday = '$mDay',
							    part_total = ".$hash['part_total'].",
							    part_reg = ".$hash['part_reg'].",
							    part_status0 = ".$hash['part_status0'].",
							    part_status1 = ".$hash['part_status1'].",
							    part_status2 = ".$hash['part_status2'].",
							    part_status3 = ".$hash['part_status3'].",
							    part_active = ".$hash['part_active'].",
							    part_account_total = ".$hash['part_account_total'].",
							    click_all = ".$hash['click_all'].",
							    click_accept = ".$hash['click_accept'].",
							    orders_all = ".$hash['orders_all'].",
							    orders_part = ".$hash['orders_part'].",
							    orders_all_sum = ".$hash['orders_all_sum'].",
							    orders_part_sum = ".$hash['orders_part_sum'].",
							    account_oper0 = ".$hash['account_oper0'].",
							    account_oper1 = ".$hash['account_oper1'].",
							    account_oper2 = ".$hash['account_oper2'].",
							    account_oper0_sum = ".$hash['account_oper0_sum'].",
							    account_oper1_sum = ".$hash['account_oper1_sum'].",
							    account_oper2_sum = ".$hash['account_oper2_sum']."
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		return true;
	}

	/**
	 * GetPartnerStatCommon
	 **/
	function GetPartnerStatCommon($mStartDate, $mEndDate)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_partner_stat_common
					 WHERE isday >= '$mStartDate' AND isday <= '$mEndDate'
				  ORDER BY isday
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerStatPartTop
	 **/
	function GetPartnerStatPartTop($mSortType, $mHow, $mToDate)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_part_orders_tmp, {$this->gDBNick}_part_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_part_orders_tmp
					       (partner_id		INTEGER NOT NULL,
							orders_count 	INTEGER NOT NULL,
							INDEX (partner_id, orders_count)
						   )
					";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_part_orders_tmp
					SELECT partner.id AS partner_id,
						   COUNT(archive_order.id) AS orders_count
					  FROM {$this->gDBNick}_partner partner
			     LEFT JOIN {$this->gDBNick}_archive_order archive_order
						ON archive_order.partner_id = partner.id
					   AND archive_order.status = 3
					   AND archive_order.date_time >= '$mToDate'
				  GROUP BY partner.id
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_part_stat_tmp
					       (partner_id		INTEGER NOT NULL,
							click_all 		INTEGER NOT NULL,
							click_accept 	INTEGER NOT NULL,
							INDEX (partner_id, click_all, click_accept)
						   )
					";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "INSERT INTO {$this->gDBNick}_part_stat_tmp
					SELECT partner.id AS partner_id,
						   SUM(partner_stat_days.click_all) AS click_all, SUM(partner_stat_days.click_accept) AS click_accept
					  FROM {$this->gDBNick}_partner partner
			     LEFT JOIN {$this->gDBNick}_partner_stat_days partner_stat_days
						ON partner_stat_days.partner_id = partner.id
		  			   AND partner_stat_days.isday >= '$mToDate'
				  GROUP BY partner.id
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$order = '';
		switch ( $mSortType )
		{
			case '0' :
				$order = ' ORDER BY orders_count DESC';
				break;
			case '1' :
				$order = ' ORDER BY click_all DESC';
				break;
			case '2' :
				$order = ' ORDER BY click_accept DESC';
				break;
			case '3' :
				$order = ' ORDER BY click_to_accept DESC';
				break;
			case '4' :
				$order = ' ORDER BY click_to_buy DESC';
				break;
		}

		$command = "SELECT partner.id AS partner_id, partner.login, partner.name,
						   part_orders_tmp.orders_count,
						   part_stat_tmp.click_all, part_stat_tmp.click_accept,
						   IF(click_all > 0, click_accept/click_all*100, 0) AS click_to_accept,
						   IF(orders_count > 0, click_accept/orders_count, 0) AS click_to_buy
					  FROM {$this->gDBNick}_partner partner,
						   {$this->gDBNick}_part_stat_tmp part_stat_tmp,
			     		   {$this->gDBNick}_part_orders_tmp part_orders_tmp
					 WHERE partner.id = part_stat_tmp.partner_id
					   AND partner.id = part_orders_tmp.partner_id
				  		   $order
					 LIMIT $mHow
			   	   ";
		return $this->gDB->DoQuery(__LINE__, $command);

	}

	/**
	 * GetStatClickFull
	 **/
	function GetStatClickFull($mPartnerID)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_partner_stat
					 WHERE partner_id = $mPartnerID
				  ORDER BY date_time DESC
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerStatAll
	 **/
	function GetPartnerStatAll($mStartDateTime, $mEndDateTime, $mPartFilter, $mPartIDs)
	{
		$filter = '';
		if ( $mPartFilter == 1 )
		{
			$filter = ' AND partner.id IN ('.$mPartIDs.')';
		}

		$command = "SELECT partner_stat.partner_id, partner_stat.date_time, partner_stat.ip, partner_stat.from_page,
						   IF(partner_stat.cookie_id = -1, -1, IF(partner_stat.cookie_id = 0, 0,
						   IF(partner_stat.cookie_id = partner_stat.partner_id, 1, 2))) AS click_accept,
						   partner.login AS partner_login
		    	  	  FROM {$this->gDBNick}_partner_stat partner_stat,
						   {$this->gDBNick}_partner partner
					 WHERE partner_stat.partner_id = partner.id
					   AND partner_stat.date_time >= '$mStartDateTime'
					   AND partner_stat.date_time <= '$mEndDateTime'
					 	   $filter
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerStatDays
	 **/
	function GetPartnerStatDays($mStartDate, $mEndDate, $mPartFilter, $mPartIDs)
	{
		$filter = '';
		if ( $mPartFilter == 1 )
		{
			$filter = ' AND partner.id IN ('.$mPartIDs.')';
		}

		$command = "SELECT partner_stat_days.partner_id, partner_stat_days.isday, partner_stat_days.click_all, partner_stat_days.click_accept,
						   IF(partner_stat_days.click_all > 0, partner_stat_days.click_accept/partner_stat_days.click_all*100, 0) AS click_to_accept,
						   partner.login AS partner_login
		    	  	  FROM {$this->gDBNick}_partner_stat_days partner_stat_days,
						   {$this->gDBNick}_partner partner
					 WHERE partner_stat_days.partner_id = partner.id
					   AND partner_stat_days.isday >= '$mStartDate'
					   AND partner_stat_days.isday <= '$mEndDate'
					 	   $filter
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerStatMonth
	 **/
	function GetPartnerStatMonth($mStartDate, $mEndDate, $mPartFilter, $mPartIDs)
	{
		$filter = '';
		if ( $mPartFilter == 1 )
		{
			$filter = ' AND partner.id IN ('.$mPartIDs.')';
		}

		$command = "SELECT partner_stat_month.partner_id, partner_stat_month.ismonth, partner_stat_month.click_all, partner_stat_month.click_accept,
						   IF(partner_stat_month.click_all > 0, partner_stat_month.click_accept/partner_stat_month.click_all*100, 0) AS click_to_accept,
						   partner.login AS partner_login
		    	  	  FROM {$this->gDBNick}_partner_stat_month partner_stat_month,
						   {$this->gDBNick}_partner partner
					 WHERE partner_stat_month.partner_id = partner.id
					   AND partner_stat_month.ismonth >= '$mStartDate'
					   AND partner_stat_month.ismonth <= '$mEndDate'
					 	   $filter
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatClickDays
	 **/
	function GetStatClickDays($mPartnerID)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_partner_stat_days
					 WHERE partner_id = $mPartnerID
				  ORDER BY isday DESC
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatClickMonth
	 **/
	function GetStatClickMonth($mPartnerID)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_partner_stat_month
					 WHERE partner_id = $mPartnerID
				  ORDER BY ismonth DESC
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetPartnerAccountLog
	 **/
	function GetPartnerAccountLog($mID)
	{
		$command = "SELECT partner_account.*, partner.login, partner.name
			    	  FROM {$this->gDBNick}_partner_account partner_account
				 LEFT JOIN {$this->gDBNick}_partner partner
		                ON partner_account.partner_id = partner.id
					 WHERE partner_account.id > '$mID'
				  ORDER BY partner_account.id
				   ";


		return $this->gDB->DoQuery(__LINE__, $command);
	}



	//***************************************************************************************************
	//
	// 									   		Additional: Melbis Shop Agent
	//
	//***************************************************************************************************

	/**
	 * GetShopLog
	 **/
	function GetShopLog($mStartTime, $mEndTime)
	{
		$command = "SELECT *
		    	  	  FROM {$this->gDBNick}_shop_stat
					 WHERE date_time >= '$mStartTime' AND date_time < '$mEndTime'
				  ORDER BY date_time ASC
				   ";
		return $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatVisitorDay
	 **/
	function GetStatVisitorDay($mStartDay, $mEndDay)
	{
		$command = "SELECT *
					  FROM {$this->gDBNick}_ss_visitor_day
					 WHERE isday >= '$mStartDay' AND isday <= '$mEndDay'
				  ORDER BY isday
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatVisitorWeek
	 **/
	function GetStatVisitorWeek($mToWeek)
	{
		$command = "SELECT *
					  FROM {$this->gDBNick}_ss_visitor_week
					 WHERE isweek <= '$mToWeek'
				  ORDER BY isweek
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatVisitorMonth
	 **/
	function GetStatVisitorMonth($mToMonth)
	{
		$command = "SELECT *
					  FROM {$this->gDBNick}_ss_visitor_month
					 WHERE ismonth <= '$mToMonth'
				  ORDER BY ismonth
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatVisitorInfoDay
	 **/
	function GetStatVisitorInfoDay($mType, $mStartDay, $mEndDay, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT $mType, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_visitor
					 WHERE ( isday >= '$mStartDay' AND isday <= '$mEndDay' )
					   AND $mType <> ''
				  GROUP BY $mType
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$domen = ( $mType == 'keyword' ) ? ', ss_visitor.domen ' : '';
		$command = "SELECT ss_visitor.isday $domen, ss_visitor.$mType, SUM(ss_visitor.how) AS how
					  FROM {$this->gDBNick}_ss_visitor ss_visitor,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.$mType = ss_visitor.$mType
					   AND ( isday >= '$mStartDay' AND isday <= '$mEndDay' )
				  GROUP BY ss_visitor.isday, ss_visitor.$mType $domen
				  ORDER BY ss_visitor.isday
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatVisitorInfoWeek
	 **/
	function GetStatVisitorInfoWeek($mType, $mToWeek, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT $mType, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_visitor
					 WHERE isweek <= '$mToWeek'
					   AND $mType <> ''
				  GROUP BY $mType
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$domen = ( $mType == 'keyword' ) ? ', ss_visitor.domen ' : '';
		$command = "SELECT ss_visitor.isweek $domen, ss_visitor.$mType, SUM(ss_visitor.how) AS how
					  FROM {$this->gDBNick}_ss_visitor ss_visitor,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.$mType = ss_visitor.$mType
					   AND isweek <= '$mToWeek'
				  GROUP BY ss_visitor.isweek, ss_visitor.$mType $domen
				  ORDER BY ss_visitor.isweek
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatVisitorInfoMonth
	 **/
	function GetStatVisitorInfoMonth($mType, $mToMonth, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT $mType, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_visitor
					 WHERE ismonth <= '$mToMonth'
					   AND $mType <> ''
				  GROUP BY $mType
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$domen = ( $mType == 'keyword' ) ? ', ss_visitor.domen ' : '';
		$command = "SELECT ss_visitor.ismonth $domen, ss_visitor.$mType, SUM(ss_visitor.how) AS how
					  FROM {$this->gDBNick}_ss_visitor ss_visitor,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.$mType = ss_visitor.$mType
					   AND ismonth <= '$mToMonth'
				  GROUP BY ss_visitor.ismonth, ss_visitor.$mType $domen
				  ORDER BY ss_visitor.ismonth
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatTopicDay
	 **/
	function GetStatTopicDay($mStartDay, $mEndDay, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT id, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_topic
					 WHERE ( isday >= '$mStartDay' AND isday <= '$mEndDay' )
				  GROUP BY id
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_topic.isday, topic.name, ss_topic.id, SUM(ss_topic.how) AS how
					  FROM {$this->gDBNick}_ss_topic ss_topic,
		                   {$this->gDBNick}_topic topic,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.id = ss_topic.id
					   AND ss_topic.id = topic.id
					   AND ( ss_topic.isday >= '$mStartDay' AND ss_topic.isday <= '$mEndDay' )
				  GROUP BY ss_topic.isday, ss_topic.id
				  ORDER BY ss_topic.isday
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatTopicWeek
	 **/
	function GetStatTopicWeek($mToWeek, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT id, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_topic
					 WHERE isweek <= '$mToWeek'
				  GROUP BY id
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_topic.isweek, topic.name, ss_topic.id, SUM(ss_topic.how) AS how
					  FROM {$this->gDBNick}_ss_topic ss_topic,
		                   {$this->gDBNick}_topic topic,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.id = ss_topic.id
					   AND ss_topic.id = topic.id
					   AND ss_topic.isweek <= '$mToWeek'
				  GROUP BY ss_topic.isweek, ss_topic.id
				  ORDER BY ss_topic.isweek
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatTopicMonth
	 **/
	function GetStatTopicMonth($mToMonth, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT id, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_topic
					 WHERE ismonth <= '$mToMonth'
				  GROUP BY id
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_topic.ismonth, topic.name, ss_topic.id, SUM(ss_topic.how) AS how
					  FROM {$this->gDBNick}_ss_topic ss_topic,
		                   {$this->gDBNick}_topic topic,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.id = ss_topic.id
					   AND ss_topic.id = topic.id
					   AND ss_topic.ismonth <= '$mToMonth'
				  GROUP BY ss_topic.ismonth, ss_topic.id
				  ORDER BY ss_topic.ismonth
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatGoodsDay
	 **/
	function GetStatGoodsDay($mStartDay, $mEndDay, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT id, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_store
					 WHERE ( isday >= '$mStartDay' AND isday <= '$mEndDay' )
				  GROUP BY id
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_store.isday, store.name, ss_store.id, SUM(ss_store.how) AS how
					  FROM {$this->gDBNick}_ss_store ss_store,
		                   {$this->gDBNick}_store store,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.id = ss_store.id
					   AND ss_store.id = store.id
					   AND ( ss_store.isday >= '$mStartDay' AND ss_store.isday <= '$mEndDay' )
				  GROUP BY ss_store.isday, ss_store.id
				  ORDER BY ss_store.isday
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatGoodsWeek
	 **/
	function GetStatGoodsWeek($mToWeek, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT id, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_store
					 WHERE isweek <= '$mToWeek'
				  GROUP BY id
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_store.isweek, store.name, ss_store.id, SUM(ss_store.how) AS how
					  FROM {$this->gDBNick}_ss_store ss_store,
		                   {$this->gDBNick}_store store,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.id = ss_store.id
					   AND ss_store.id = store.id
					   AND ss_store.isweek <= '$mToWeek'
				  GROUP BY ss_store.isweek, ss_store.id
				  ORDER BY ss_store.isweek
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatGoodsMonth
	 **/
	function GetStatGoodsMonth($mToMonth, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT id, SUM(how) AS total
					  FROM {$this->gDBNick}_ss_store
					 WHERE ismonth <= '$mToMonth'
				  GROUP BY id
		          ORDER BY total DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_store.ismonth, store.name, ss_store.id, SUM(ss_store.how) AS how
					  FROM {$this->gDBNick}_ss_store ss_store,
		                   {$this->gDBNick}_store store,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.id = ss_store.id
					   AND ss_store.id = store.id
					   AND ss_store.ismonth <= '$mToMonth'
				  GROUP BY ss_store.ismonth, ss_store.id
				  ORDER BY ss_store.ismonth
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}



	/**
	 * GetStatOrderInfoWeek
	 **/
	function GetStatOrderInfoWeek($mType, $mToWeek, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT $mType, COUNT($mType) AS how
					  FROM {$this->gDBNick}_ss_order
					 WHERE isweek <= '$mToWeek'
					   AND $mType <> ''
				  GROUP BY $mType
		          ORDER BY how DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$domen = ( $mType == 'keyword' ) ? ', ss_order.domen ' : '';
		$command = "SELECT ss_order.isweek $domen, ss_order.$mType, COUNT(ss_order.$mType) AS how, SUM(ss_order.suma) AS suma
					  FROM {$this->gDBNick}_ss_order ss_order,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.$mType = ss_order.$mType
					   AND isweek <= '$mToWeek'
				  GROUP BY ss_order.isweek, ss_order.$mType $domen
				  ORDER BY ss_order.isweek
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatOrderInfoMonth
	 **/
	function GetStatOrderInfoMonth($mType, $mToMonth, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT $mType, COUNT($mType) AS how
					  FROM {$this->gDBNick}_ss_order
					 WHERE ismonth <= '$mToMonth'
					   AND $mType <> ''
				  GROUP BY $mType
		          ORDER BY how DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$domen = ( $mType == 'keyword' ) ? ', ss_order.domen ' : '';
		$command = "SELECT ss_order.ismonth $domen, ss_order.$mType, COUNT(ss_order.$mType) AS how, SUM(ss_order.suma) AS suma
					  FROM {$this->gDBNick}_ss_order ss_order,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.$mType = ss_order.$mType
					   AND ismonth <= '$mToMonth'
				  GROUP BY ss_order.ismonth, ss_order.$mType $domen
				  ORDER BY ss_order.ismonth
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatShopSearchDay
	 **/
	function GetStatShopSearchDay($mStartDay, $mEndDay, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT one_word, COUNT(one_word) AS how
					  FROM {$this->gDBNick}_ss_search
					 WHERE ( isday >= '$mStartDay' AND isday <= '$mEndDay' )
				  GROUP BY one_word
		          ORDER BY how DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_search.isday, ss_search.one_word, ss_search.phrase, COUNT(ss_search.one_word) AS how
					  FROM {$this->gDBNick}_ss_search ss_search,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.one_word = ss_search.one_word
					   AND ( ss_search.isday >= '$mStartDay' AND ss_search.isday <= '$mEndDay' )
				  GROUP BY ss_search.isday, ss_search.one_word, ss_search.phrase
				  ORDER BY ss_search.isday
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatShopSearchWeek
	 **/
	function GetStatShopSearchWeek($mToWeek, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT one_word, COUNT(one_word) AS how
					  FROM {$this->gDBNick}_ss_search
					 WHERE isweek <= '$mToWeek'
				  GROUP BY one_word
		          ORDER BY how DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_search.isweek, ss_search.one_word, ss_search.phrase, COUNT(ss_search.one_word) AS how
					  FROM {$this->gDBNick}_ss_search ss_search,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.one_word = ss_search.one_word
					   AND isweek <= '$mToWeek'
				  GROUP BY ss_search.isweek, ss_search.one_word, ss_search.phrase
				  ORDER BY ss_search.isweek
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}

	/**
	 * GetStatShopSearchMonth
	 **/
	function GetStatShopSearchMonth($mToMonth, $mLimit)
	{
		$command = "DROP TABLE IF EXISTS {$this->gDBNick}_stat_tmp";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "CREATE TEMPORARY TABLE {$this->gDBNick}_stat_tmp
					SELECT one_word, COUNT(one_word) AS how
					  FROM {$this->gDBNick}_ss_search
					 WHERE ismonth <= '$mToMonth'
				  GROUP BY one_word
		          ORDER BY how DESC
		             LIMIT $mLimit
				   ";
		$this->gDB->DoQuery(__LINE__, $command);

		$command = "SELECT ss_search.ismonth, ss_search.one_word, ss_search.phrase, COUNT(ss_search.one_word) AS how
					  FROM {$this->gDBNick}_ss_search ss_search,
						   {$this->gDBNick}_stat_tmp stat_tmp
					 WHERE stat_tmp.one_word = ss_search.one_word
					   AND ismonth <= '$mToMonth'
				  GROUP BY ss_search.ismonth, ss_search.one_word, ss_search.phrase
				  ORDER BY ss_search.ismonth
				   ";

		return  $this->gDB->DoQuery(__LINE__, $command);
	}


	//***************************************************************************************************
	//
	// 									   		Additional: Melbis Shop Spider
	//
	//***************************************************************************************************

	/**
	 * UpdatePriceList
	 **/
	function UpdatePriceList($mCode, $mHow, $mActive, $mStatus, $mPartProc, $mProvPrice, $mProvPriceBase, $mProvCurrID,
							 $mPrice, $mPriceBase, $mPriceCurrID, $mDiscPrice, $mDiscPriceBase, $mDiscCurrID, $mInXML)
	{
		$command = "SELECT id
	    	  	  	  FROM {$this->gDBNick}_store
				     WHERE code = '$mCode'
				       ";
		$sth = $this->gDB->DoQuery(__LINE__, $command);

		// Goods exists?
		$hash = $this->FetchHash($sth);
		if ( $hash['id'] != 0 )
		{
			$set = '';
			if ( strlen($mHow) > 0 ) $set .= " how = '".($mHow*1)."',";
			if ( strlen($mActive) > 0 ) $set .= " active = '".($mActive*1)."',";
			if ( strlen($mStatus) > 0 ) $set .= " status = '".($mStatus*1)."',";
			if ( strlen($mInXML) > 0 ) $set .= " in_xml = '".($mInXML*1)."',";
			if ( strlen($mPartProc) > 0 ) $set .= " part_proc = '".($mPartProc*1)."',";
			if ( strlen($mProvPrice) > 0 )
			{
				$set .= " prov_price = '$mProvPrice', prov_price_base = '$mProvPriceBase', pprice_curr_id = '$mProvCurrID',";
			}
			if ( strlen($mPrice) > 0 )
			{
				$set .= " price = '$mPrice', price_base = '$mPriceBase', price_curr_id = '$mPriceCurrID',";
			}
			if ( strlen($mDiscPrice) > 0 )
			{
				$set .= " disc_price = '$mDiscPrice', disc_price_base = '$mDiscPriceBase', dprice_curr_id = '$mDiscCurrID',";
			}
			$set = rtrim($set, ',');

			if ( strlen($set) == 0 ) return false;

			$command = "UPDATE {$this->gDBNick}_store
						   SET $set
						 WHERE code = '$mCode'
				   	   ";
			$this->gDB->DoQuery(__LINE__, $command);

		}
		else
		{
			$set = '';
			if ( strlen($mHow) > 0 ) $set .= " how = '".($mHow*1)."',";
			if ( strlen($mStatus) > 0 ) $set .= " status = '".($mStatus*1)."',";
			if ( strlen($mPrice) > 0 )
			{
				$set .= " price = '$mPrice', price_base = '$mPriceBase', price_curr_id = '$mPriceCurrID',";
			}
			$set = rtrim($set, ',');

			if ( strlen($set) == 0 ) return false;

			$command = "UPDATE {$this->gDBNick}_store_add
						   SET $set
						 WHERE code = '$mCode'
				   	   ";
			$this->gDB->DoQuery(__LINE__, $command);
		}

	return true;

	}



}

?>